#include "temperature.h"
#include "ultralcd.h"
#ifdef ULTRA_LCD
#include "Marlin.h"
#include "language.h"
#include "cardreader.h"
#include "temperature.h"
#include "stepper.h"
#include "ConfigurationStore.h"

#include "Configuration.h"

/* Configuration settings */
/* 2014/08/12
int plaPreheatHotendTemp;
int plaPreheatHPBTemp;
int plaPreheatFanSpeed;

int absPreheatHotendTemp;
int absPreheatHPBTemp;
int absPreheatFanSpeed;
*/
/* !Configuration settings */

//Function pointer to menu functions.
typedef void (*menuFunc_t)();

uint8_t lcd_status_message_level;

//2014/04/23
//#if LANGUAGE_CHOICE == 1 //japanese
//char lcd_status_message[LCD_WIDTH*2+1] = WELCOME_MSG;
//#else
char lcd_status_message[LCD_WIDTH+1] = WELCOME_MSG;
//#endif

#ifdef DOGLCD
#include "dogm_lcd_implementation.h"
#else
#include "ultralcd_implementation_hitachi_HD44780.h"
#endif

/** forward declerations **/

//void copy_and_scalePID_i();
//void copy_and_scalePID_d();

/* Different menus */
static void lcd_status_screen();
#ifdef ULTIPANEL
static void lcd_main_menu();
static void lcd_tune_menu();
//static void lcd_prepare_menu();
static void lcd_move_menu();
static void lcd_control_menu();
//static void lcd_control_temperature_menu();
//static void lcd_control_temperature_preheat_pla_settings_menu();
//static void lcd_control_temperature_preheat_abs_settings_menu();
static void lcd_control_motion_menu();
static void lcd_control_retract_menu();
static void lcd_sdcard_menu();

static void lcd_quick_feedback();//Cause an LCD refresh, and give the user visual or audiable feedback that something has happend

/* Different types of actions that can be used in menuitems. */
static void menu_action_back(menuFunc_t data);
static void menu_action_submenu(menuFunc_t data);
static void menu_action_gcode(const char* pgcode);
static void menu_action_function(menuFunc_t data);
static void menu_action_sdfile(const char* filename, char* longFilename);
static void menu_action_sddirectory(const char* filename, char* longFilename);
static void menu_action_setting_edit_bool(const char* pstr, bool* ptr);
static void menu_action_setting_edit_int3(const char* pstr, int* ptr, int minValue, int maxValue);
static void menu_action_setting_edit_float3(const char* pstr, float* ptr, float minValue, float maxValue);
static void menu_action_setting_edit_float32(const char* pstr, float* ptr, float minValue, float maxValue);
static void menu_action_setting_edit_float5(const char* pstr, float* ptr, float minValue, float maxValue);
static void menu_action_setting_edit_float51(const char* pstr, float* ptr, float minValue, float maxValue);

//2014/07/18
extern unsigned int MENU_PAGE;
extern unsigned int SD_FILE_COUNT;
extern int PUSH_SW;
extern bool SD_NOT_PRINT = true;
extern bool SYS_COLD_ERR;
extern bool FINISH_PROC;

static void lcd_filament_heating();
static void lcd_nozzle_heating();
static void lcd_bed_heating();
static void lcd_preheat_menu();
static void lcd_no_action();
static void lcd_stop_heat();
static void lcd_print_screen();
static void lcd_cls();
static void lcd_print_finish();
static void lcd_move_x_menu();
static void lcd_move_x_menu();
static void lcd_move_z_menu();
static void lcd_xmove50();
static void lcd_ymove50();
static void lcd_zmove10();
static void lcd_sdcard_stop();
static void lcd_print_finish_proc();
static void lcd_temp_monitor();

static bool pre_heat_flg = false;
static int  bak_encpos = 0;
static bool sd_print_flg = false;
static int sd_file_pos_bak = -1;
static bool sd_detect = false;
static int sd_curpos = 0;
static bool lcd_cls_flg = false;
static int sd_file_pos = 0;
static int thotend_bak = -1;
static int tbed_bak = -1;
static int t_xmove = 1;
static int t_ymove = 1;
static int t_zmove = 1;
static int move_flg = -1;
static bool xcmd_flg = false;
static bool ycmd_flg = false;
static bool zcmd_flg = false;

static int mente[10];
static int mente_no = 0;
static int print_speed = 100;
static int gear_speed = 100;
static int adjust_fan_speed = 100;
static bool exec_print = false;
static bool adjust_flg = false;
static bool folder_flg = false;
static int R_SW_CNT = 0;
static int Fcnt = 0;
static int FldCnt = 0;
static bool no_card_flg = false;

//2014/08/26
static int RELEASE_CNT = 0;
static int DOWN_CNT = 0;
static int UP_CNT = 0;
static int LEFT_CNT = 0;
static int RIGHT_CNT = 0;
static int ENTER_CNT = 0;
static int HOME_CNT = 0;
static bool HEAT_OFF = true;
static bool BED_OFF = true;

///


//2014/04/23
/*
static void utf_ascii(char *);
//extern void utf_uni(char *);

static void utf_ascii(char *s){
    byte i=0;
    byte j=0;
    while(s[i]!='\0'){
        if((byte)s[i]==0xEF){
            if((byte)s[i+1]==0xBE) s[i+2] += 0x40;
            i+=2;
        }
        s[j]= s[i];
        i++;
        j++;
    }
    s[j]='\0';
}
///
*/

extern void move_home(void);
extern void move_command(int pos);



#define ENCODER_STEPS_PER_MENU_ITEM 5

/* Helper macros for menus */
#define START_MENU() do { \
    if (encoderPosition > 0x8000) encoderPosition = 0; \
    if (encoderPosition / ENCODER_STEPS_PER_MENU_ITEM < currentMenuViewOffset) currentMenuViewOffset = encoderPosition / ENCODER_STEPS_PER_MENU_ITEM;\
    uint8_t _lineNr = currentMenuViewOffset, _menuItemNr; \
    for(uint8_t _drawLineNr = 0; _drawLineNr < LCD_HEIGHT; _drawLineNr++, _lineNr++) { \
        _menuItemNr = 0;
#define MENU_ITEM(type, label, args...) do { \
    if (_menuItemNr == _lineNr) { \
        if (lcdDrawUpdate) { \
            const char* _label_pstr = PSTR(label); \
            if ((encoderPosition / ENCODER_STEPS_PER_MENU_ITEM) == _menuItemNr) { \
                lcd_implementation_drawmenu_ ## type ## _selected (_drawLineNr, _label_pstr , ## args ); \
            }else{\
                lcd_implementation_drawmenu_ ## type (_drawLineNr, _label_pstr , ## args ); \
            }\
        }\
        if (LCD_CLICKED && (encoderPosition / ENCODER_STEPS_PER_MENU_ITEM) == _menuItemNr) {\
            lcd_quick_feedback(); \
            menu_action_ ## type ( args ); \
            return;\
        }\
    }\
    _menuItemNr++;\
} while(0)
#define MENU_ITEM_DUMMY() do { _menuItemNr++; } while(0)
#define MENU_ITEM_EDIT(type, label, args...) MENU_ITEM(setting_edit_ ## type, label, PSTR(label) , ## args )
#define MENU_ITEM_EDIT_CALLBACK(type, label, args...) MENU_ITEM(setting_edit_callback_ ## type, label, PSTR(label) , ## args )
#define END_MENU() \
    if (encoderPosition / ENCODER_STEPS_PER_MENU_ITEM >= _menuItemNr) encoderPosition = _menuItemNr * ENCODER_STEPS_PER_MENU_ITEM - 1; \
    if ((uint8_t)(encoderPosition / ENCODER_STEPS_PER_MENU_ITEM) >= currentMenuViewOffset + LCD_HEIGHT) { currentMenuViewOffset = (encoderPosition / ENCODER_STEPS_PER_MENU_ITEM) - LCD_HEIGHT + 1; lcdDrawUpdate = 1; _lineNr = currentMenuViewOffset - 1; _drawLineNr = -1; } \
    } } while(0)

/** Used variables to keep track of the menu */
volatile uint8_t buttons;//Contains the bits of the currently pressed buttons.

uint8_t currentMenuViewOffset;              /* scroll offset in the current menu */
uint32_t blocking_enc;
uint8_t lastEncoderBits;
int8_t encoderDiff; /* encoderDiff is updated from interrupt context and added to encoderPosition every LCD update */
uint32_t encoderPosition;
#if (SDCARDDETECT > -1)
bool lcd_oldcardstatus;
#endif
#endif//ULTIPANEL

menuFunc_t currentMenu = lcd_main_menu; //2014/07/10 lcd_status_screen; /* function pointer to the currently active menu */
uint32_t lcd_next_update_millis;
//uint8_t lcd_status_update_delay;
uint8_t lcdDrawUpdate = 2;                  /* Set to none-zero when the LCD needs to draw, decreased after every draw. Set to 2 in LCD routines so the LCD gets atleast 1 full redraw (first redraw is partial) */

//prevMenu and prevEncoderPosition are used to store the previous menu location when editing settings.
menuFunc_t prevMenu = NULL;
uint16_t prevEncoderPosition;
//Variables used when editing values.
const char* editLabel;
void* editValue;
int32_t minEditValue, maxEditValue;
menuFunc_t callbackFunc;

// placeholders for Ki and Kd edits
float raw_Ki, raw_Kd;


/* Main status screen. It's up to the implementation specific part to show what is needed. As this is very display dependend */
/*
static void lcd_status_screen()
{
    //2014/04/19
    MENU_PAGE = 0;
    
    if (lcd_status_update_delay)
        lcd_status_update_delay--;
    else
        lcdDrawUpdate = 1;
    if (lcdDrawUpdate)
    {
        lcd_implementation_status_screen();
        lcd_status_update_delay = 10;   // redraw the main screen every second. This is easier then trying keep track of all things that change on the screen 
    }
#ifdef ULTIPANEL

    if (LCD_CLICKED)
    {
        SERIAL_PROTOCOLPGM("status screen()"); 
        currentMenu = lcd_main_menu;
        lcd_quick_feedback();
    }

    feedmultiply += int(encoderPosition);
    encoderPosition = 0;
    if (feedmultiply < 10)
        feedmultiply = 10;
    if (feedmultiply > 999)
        feedmultiply = 999;
#endif//ULTIPANEL
}
*/


#ifdef ULTIPANEL

static void lcd_cls()
{
    lcd.setCursor(0, 0);
    lcd.print("                ");
    lcd.setCursor(0, 1);
    lcd.print("                ");
    lcd.setCursor(-4, 2);
    lcd.print("                ");
    lcd.setCursor(-4, 3);
    lcd.print("                ");
}

static void lcd_print_finish()
{
    MENU_PAGE = 111;
    
    if (encoderPosition < 5)
        encoderPosition = 5;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
    
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_PRINT_END);
    lcd.setCursor(0, 1);
    lcd.print(MSG_SAME_PRINT);
    lcd.setCursor(-3, 2);
    lcd.print("\xb2\xb2\xb4");        //
    lcd.setCursor(-3, 3);
    lcd.print("\xca\xb2");            //ʎ
    
    if (encoderPosition == 10)
    {
        lcd.setCursor(-4, 2);
        lcd.print('>');
        lcd.setCursor(-4, 3);
        lcd.print(' ');
    }
    if (encoderPosition == 15)
    {
        lcd.setCursor(-4, 2);
        lcd.print(' ');
        lcd.setCursor(-4, 3);
        lcd.print('>');
    }
}


static void lcd_print_finish_proc()
{
    MENU_PAGE = 999;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
        
        lcd.setCursor(0, 0);
        lcd.print(MSG_PRINT_END);
        lcd.setCursor(0, 1);
        lcd.print(MSG_PRINT_FIN_PROC);    //؎ ؎
        
        FINISH_PROC = true;
        target_temperature[0] = 0;
        target_temperature_bed = 0;
        
        st_synchronize();
        quickStop();
        card.closefile();
        
        enquecommand_P(PSTR("M22"));
        
    }
    
    while(true)
    {
        lcd.setCursor(0, 0);
        lcd.print(MSG_PRINT_END);
        lcd.setCursor(0, 1);
        lcd.print(MSG_PRINT_FIN_PROC);    //؎ ؎


        if (SD_NOT_PRINT == true)
        {
            MENU_PAGE = 998;
            //sd_print_flg = false;
            //SERIAL_PROTOCOLLN("exit");
            break;
        }
    }
    //currentMenu = lcd_main_menu;
    //lcd_quick_feedback();
}
static void lcd_print_adjust_nozzle()
{
    MENU_PAGE = 121;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_CHANGE);
    lcd.setCursor(0, 1);
    lcd.print(MSG_A_NOZZLE);                              // Ɏގ
    lcd.print(LCD_STR_UPLEVEL[0]);
    lcd.setCursor(-4, 2);
    lcd.print(MSG_A_TEMP);                                // ݎĎ
    //lcd.setCursor(2, 2);
    lcd.print(itostr3(int(target_temperature[0])));
    //lcd.print(itostr3(int(degTargetHotend(0) + 0.5)));  // value
    lcd.print('/');
    lcd.print(itostr3(HEAT_TEMP_MAX));                    // MAX
    lcd.print(LCD_STR_DEGREE);
    lcd.print('C');
    lcd.setCursor(4, 3);
    lcd.print(LCD_STR_DOWNLEVEL[0]);
}

static void lcd_print_adjust_bed_temp()
{
    MENU_PAGE = 122;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
        //SERIAL_PROTOCOLPGM("lcd_print_adjust_bed_temp clear"); 
        //SERIAL_PROTOCOLLN("");
    }
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_CHANGE);
    lcd.setCursor(0, 1);
    lcd.print("\xcd\xde\xaf\xc4\xde   ");                 // ͎ގĎ
    lcd.print(LCD_STR_UPLEVEL[0]);
    lcd.setCursor(-4, 2);
    lcd.print("\xb5\xdd\xc4\xde  ");                      // ݎĎ
    //lcd.setCursor(2, 2);
    lcd.print(itostr3(int(target_temperature_bed)));
    //lcd.print(itostr3(int(degTargetBed() + 0.5)));      // value
    lcd.print('/');
    lcd.print(itostr3(BED_TEMP_MAX));                     // MAX
    lcd.print(LCD_STR_DEGREE);
    lcd.print('C');
    lcd.setCursor(4, 3);
    lcd.print(LCD_STR_DOWNLEVEL[0]);
}

static void lcd_print_adjust_print_speed()
{
    MENU_PAGE = 123;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_CHANGE);
    lcd.setCursor(0, 1);
    lcd.print("\xcc\xdf\xd8\xdd\xc4   ");                 // ̎ߎ؎ݎ
    lcd.print(LCD_STR_UPLEVEL[0]);
    lcd.setCursor(-4, 2);
    lcd.print("\xbd\xcb\xdf\xb0\xc4\xde ");               // ˎߎĎ
    //lcd.setCursor(2, 2);
    lcd.print(itostr3(print_speed));                      // value
    lcd.print('/');
    lcd.print(itostr3(PRINT_SPEED_MAX));
    lcd.print("%");
    //lcd.print("300%");                                    // MAX
    lcd.setCursor(4, 3);
    lcd.print(LCD_STR_DOWNLEVEL[0]);
}

static void lcd_print_adjust_gear_speed()
{
    MENU_PAGE = 124;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_CHANGE);
    lcd.setCursor(0, 1);
    lcd.print("\xb7\xde\xb1     ");                       // ގ
    lcd.print(LCD_STR_UPLEVEL[0]);
    lcd.setCursor(-4, 2);
    lcd.print("\xbd\xcb\xdf\xb0\xc4\xde ");               // ˎߎĎ
    //lcd.setCursor(2, 2);
    lcd.print(itostr3(gear_speed));                       // value
    lcd.print('/');
    lcd.print("150%");                                    // MAX
    lcd.setCursor(4, 3);
    lcd.print(LCD_STR_DOWNLEVEL[0]);
}

static void lcd_print_adjust_fan_speed()
{
    MENU_PAGE = 125;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_CHANGE);
    lcd.setCursor(0, 1);
    lcd.print("\xcc\xa7\xdd     ");                       // ̎
    lcd.print(LCD_STR_UPLEVEL[0]);
    lcd.setCursor(-4, 2);
    lcd.print("\xbd\xcb\xdf\xb0\xc4\xde ");               // ˎߎĎ
    //lcd.setCursor(2, 2);
    lcd.print(itostr3(adjust_fan_speed));                 // value
    lcd.print('/');
    lcd.print("100%");                                    // MAX
    lcd.setCursor(4, 3);
    lcd.print(LCD_STR_DOWNLEVEL[0]);
}

static void lcd_print_adjust_print_finish()
{
    MENU_PAGE = 126;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
    
    if (encoderPosition < 5)
        encoderPosition = 5;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_PRINT_END);
    lcd.setCursor(1, 1);
    lcd.print("\xb2\xb2\xb4");        //
    lcd.setCursor(-3, 2);
    lcd.print("\xca\xb2");            //ʎ
    
    if (encoderPosition == 5)
    {
        lcd.setCursor(0, 1);
        lcd.print('>');
        lcd.setCursor(-4, 2);
        lcd.print(' ');
    }
    if (encoderPosition == 10)
    {
        lcd.setCursor(0, 1);
        lcd.print(' ');
        lcd.setCursor(-4, 2);
        lcd.print('>');
    }
}

static void lcd_print_adjust_menu()
{
    MENU_PAGE = 120;
    
    /*
    if (lcd_cls_flg == false)
    {
        lcd_cls();
        lcd_cls_flg = true;
    }
    */
    
    if (encoderPosition < 5)
        encoderPosition = 5;
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_PRINT_ADJUST);
    
    if (encoderPosition == 5)
    {
        lcd.setCursor(0, 1);
        lcd.print('>');
        lcd.setCursor(1, 1);
        lcd.print(MSG_NOZZLE_TEMP);
        lcd.setCursor(-4, 2);
        lcd.print(' ');
        lcd.setCursor(-3, 2);
        lcd.print(MSG_BED_TEMP);
        lcd.setCursor(-4, 3);
        lcd.print(' ');
        lcd.setCursor(-3, 3);
        lcd.print(MSG_PRINT_SPEED);
    }
    if (encoderPosition == 10)
    {
        lcd.setCursor(0, 1);
        lcd.print(' ');
        lcd.setCursor(1, 1);
        lcd.print(MSG_NOZZLE_TEMP);
        lcd.setCursor(-4, 2);
        lcd.print('>');
        lcd.setCursor(-3, 2);
        lcd.print(MSG_BED_TEMP);
        lcd.setCursor(-4, 3);
        lcd.print(' ');
        lcd.setCursor(-3, 3);
        lcd.print(MSG_PRINT_SPEED);
    }
    if (encoderPosition == 15)
    {
        lcd.setCursor(0, 1);
        lcd.print(' ');
        lcd.setCursor(1, 1);
        lcd.print(MSG_NOZZLE_TEMP);
        lcd.setCursor(-4, 2);
        lcd.print(' ');
        lcd.setCursor(-3, 2);
        lcd.print(MSG_BED_TEMP);
        lcd.setCursor(-4, 3);
        lcd.print('>');
        lcd.setCursor(-3, 3);
        lcd.print(MSG_PRINT_SPEED);
    }
    if (encoderPosition == 20)
    {
        lcd.setCursor(0, 1);
        lcd.print(' ');
        lcd.setCursor(1, 1);
        lcd.print(MSG_BED_TEMP);
        lcd.setCursor(-4, 2);
        lcd.print(' ');
        lcd.setCursor(-3, 2);
        lcd.print(MSG_PRINT_SPEED);
        lcd.setCursor(-4, 3);
        lcd.print('>');
        lcd.setCursor(-3, 3);
        lcd.print(MSG_GEAR_SPEED);
    }
    if (encoderPosition == 25)
    {
        lcd.setCursor(0, 1);
        lcd.print(' ');
        lcd.setCursor(1, 1);
        lcd.print(MSG_PRINT_SPEED);
        lcd.setCursor(-4, 2);
        lcd.print(' ');
        lcd.setCursor(-3, 2);
        lcd.print(MSG_GEAR_SPEED);
        lcd.setCursor(-4, 3);
        lcd.print('>');
        lcd.setCursor(-3, 3);
        lcd.print(MSG_FAN_SPEED);
    }
    if (encoderPosition == 30)
    {
        lcd.setCursor(0, 1);
        lcd.print(' ');
        lcd.setCursor(1, 1);
        lcd.print(MSG_GEAR_SPEED);
        lcd.setCursor(-4, 2);
        lcd.print(' ');
        lcd.setCursor(-3, 2);
        lcd.print(MSG_FAN_SPEED);
        lcd.setCursor(-4, 3);
        lcd.print('>');
        lcd.setCursor(-3, 3);
        lcd.print(MSG_PRINT_FIN);
    }
}


static void lcd_print_screen()
{
    int eof = 0;
    int dot = 0;
    
    MENU_PAGE = 110;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
        
        for(int i = 0; i < 16; i++)
        {
            if (card.filename[i] == 0x7e)
            {
                card.filename[i] = LCD_STR_TILDE[0];
                break;
            }
        }
    }
    
    lcd.setCursor(0, 0);
    lcd.print(card.filename);
    lcd.setCursor(11, 0);
    lcd.print(itostr3(card.percentDone()));
    lcd.setCursor(14, 0);
    lcd.print('%');
    
    lcd.setCursor(0, 1);
    lcd.print(MSG_NOZZLE);
    lcd.setCursor(6, 1);
    lcd.print(itostr3(int(degHotend(0) + 0.5)));
    //lcd.setCursor(10, 1);
    lcd.print('/');
    //lcd.setCursor(12, 1);
    lcd.print(itostr3(int(degTargetHotend(0) + 0.5)));
    lcd.print(LCD_STR_DEGREE);
    lcd.print('C');
    
    lcd.setCursor(-4, 2);
    lcd.print(MSG_BED);
    lcd.setCursor(2, 2);
    lcd.print(itostr3(int(degBed() + 0.5)));
    //lcd.setCursor(6, 2);
    lcd.print('/');
    //lcd.setCursor(8, 2);
    lcd.print(itostr3(int(degTargetBed() + 0.5)));
    lcd.print(LCD_STR_DEGREE);
    lcd.print('C');
    
    if (card.percentDone() > 1 )
        exec_print = true;
    
    //SERIAL_PROTOCOLPGM("print % = "); 
    //SERIAL_PROTOCOLLN(itostr3(card.percentDone()));
    
    
    if ((IS_SD_PRINTING == false) && (exec_print == true))
    {
        //lcd_sdcard_stop();
        
        sd_print_flg = false;
        lcd_cls_flg = false;
        encoderPosition = 10;
        
        lcd_quick_feedback();
        currentMenu = lcd_print_finish;
    }
    
    //lcd_status_update_delay = 10;   // redraw the main screen every second. This is easier then trying keep track of all things that change on the screen 
}


static void lcd_return_to_status()
{
    //2014/07/18
    //encoderPosition = 0;
    //currentMenu = lcd_status_screen;
    
    encoderPosition = 0;
    
    switch(MENU_PAGE)
    {
        case 110:  //print status
            /*
            if (sd_print_flg != true)
            {
                PUSH_SW = 0;
                currentMenu = lcd_print_adjust_menu;
                sd_print_flg = false;
                sd_file_pos_bak = -1;
                sd_file_pos = 0;
            }
            */
            break;
            
        case 100:
            card.setroot();
            currentMenu = lcd_main_menu;
            PUSH_SW = 0;
            sd_print_flg = false;
            sd_file_pos_bak = -1;
            sd_file_pos = 0;
            lcd_cls_flg = false;
            break;
            
        case 120:  //print adjust
        case 121:
        case 122:
        case 123:
        case 124:
        case 125:
        case 126:
            currentMenu = lcd_print_screen;
            PUSH_SW = 0;
            break;
            
        case 200:  //filament
            break;
        
        /*
        case 401:  //nozzle heating
            //lcd_cls();
            currentMenu = lcd_main_menu;
            PUSH_SW = 0;
            break;
        */
        
            
        default:
            currentMenu = lcd_main_menu;
            PUSH_SW = 0;
            no_card_flg = false;  //2014/08/25
            sd_print_flg = false;
            sd_file_pos_bak = -1;
            sd_file_pos = 0;
            lcd_cls_flg = false;
            break;
    }
    
    //SERIAL_PROTOCOLLN(MENU_PAGE); 
    //SERIAL_PROTOCOLLN("returnto status()"); 
}

/*
static void lcd_sdcard_pause()
{
    card.pauseSDPrint();
}
static void lcd_sdcard_resume()
{
    card.startFileprint();
}
*/

static void lcd_sdcard_stop()
{
    card.sdprinting = false;
    card.closefile();
    quickStop();
    if(SD_FINISHED_STEPPERRELEASE)
    {
        enquecommand_P(PSTR(SD_FINISHED_RELEASECOMMAND));
    }
    autotempShutdown();
}

/* Menu implementation */
static void lcd_main_menu()
{
    //2014/04/19
    MENU_PAGE = 1;
    //
    
    //2014/07/15
    if (encoderPosition < 5)
        encoderPosition = 5;
    ///

    //SERIAL_PROTOCOLPGM("main_manu: encoderPosition ="); 
    //SERIAL_PROTOCOLLN((int)encoderPosition);

    /*
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
    */
    
    //2014/08/25
    if (encoderPosition == 25)
    {
        START_MENU();
        MENU_ITEM(function, MSG_WATCH, lcd_main_menu);
        MENU_ITEM(submenu, MSG_CONTROL, lcd_move_z_menu);               //2014/08/25 move_x -> move_z
        MENU_ITEM(submenu, MSG_PREHEAT, lcd_preheat_menu);
        MENU_ITEM(submenu, MSG_TEMP_MON, lcd_temp_monitor);  //500
        END_MENU();
        
        lcd.setCursor(15, 0);
        lcd.print(' ');
        lcd.setCursor(15, 1);
        lcd.print(' ');
        lcd.setCursor(11, 2);
        lcd.print(' ');
        lcd.setCursor(11, 3);
        lcd.print(' ');
        
        encoderPosition = 25;
    }
    /// 2014/08/25
    
    else 
    {
        if (encoderPosition == 20)
        {
            START_MENU();
            MENU_ITEM(function, MSG_WATCH, lcd_main_menu);
            MENU_ITEM(submenu, MSG_FILAMENT, lcd_filament_heating);
            MENU_ITEM(submenu, MSG_CONTROL, lcd_move_z_menu);             //2014/08/25 move_x -> move_z
            MENU_ITEM(submenu, MSG_PREHEAT, lcd_preheat_menu);
            END_MENU();
            lcd.setCursor(15, 0);
            lcd.print(' ');
            lcd.setCursor(15, 1);
            lcd.print(' ');
            lcd.setCursor(11, 2);
            lcd.print(' ');
            lcd.setCursor(11, 3);
            lcd.print(' ');
        }
        else
        {
            START_MENU();
            MENU_ITEM(function, MSG_WATCH, lcd_main_menu);
            MENU_ITEM(submenu, MSG_PRINT, lcd_sdcard_menu);           //100
            MENU_ITEM(submenu, MSG_FILAMENT, lcd_filament_heating);   //200
            MENU_ITEM(submenu, MSG_CONTROL, lcd_move_z_menu);         //300   2014/08/25 move_x -> move_Z
            //MENU_ITEM(submenu, MSG_PREHEAT, lcd_preheat_menu);        //400
            END_MENU();
            
            lcd.setCursor(15, 0);
            lcd.print(' ');
            lcd.setCursor(15, 1);
            lcd.print(' ');
            lcd.setCursor(11, 2);
            lcd.print(' ');
            lcd.setCursor(11, 3);
            lcd.print(' ');
        }
    }
    ///

/*  2014/07/18
    if (movesplanned() || IS_SD_PRINTING)
    {
        MENU_ITEM(submenu, MSG_TUNE, lcd_tune_menu);
        
    }else{
        MENU_ITEM(submenu, MSG_PREPARE, lcd_prepare_menu);
    }
    MENU_ITEM(submenu, MSG_CONTROL, lcd_control_menu);
#ifdef SDSUPPORT
    if (card.cardOK)
    {
        if (card.isFileOpen())
        {
            if (card.sdprinting)
                MENU_ITEM(function, MSG_PAUSE_PRINT, lcd_sdcard_pause);
            else
                MENU_ITEM(function, MSG_RESUME_PRINT, lcd_sdcard_resume);
            MENU_ITEM(function, MSG_STOP_PRINT, lcd_sdcard_stop);
        }else{
            MENU_ITEM(submenu, MSG_CARD_MENU, lcd_sdcard_menu);
#if SDCARDDETECT < 1
			MENU_ITEM(gcode, MSG_CNG_SDCARD, PSTR("M21"));	// SD-card changed by user
#endif			
        }
    }else{
        MENU_ITEM(submenu, MSG_NO_CARD, lcd_sdcard_menu);
#if SDCARDDETECT < 1		
		MENU_ITEM(gcode, MSG_INIT_SDCARD, PSTR("M21"));	// Manually initialize the SD-card via user interface
#endif		
    }
#endif
    END_MENU();
*/
}

//2014/07/18
static void lcd_filament_heating()
{
    MENU_PAGE = 200;
    
    char cmd[30];
    sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(NOZZLE_TEMP)));
    
    int tHotend=int(degHotend(0) + 0.5);
    int tTarget=int(degTargetHotend(0) + 0.5);
    
    if (pre_heat_flg == false)
    {
        pre_heat_flg = true;
        
        enquecommand(cmd);
    }
    
    lcd.setCursor(0,0);
    lcd.print(MSG_FILAMENT_TITLE);
    lcd.setCursor(8,1);
    lcd.print(LCD_STR_UPLEVEL[0]);
    lcd.setCursor(10,1);
    lcd.print(MSG_REMOVE);
    lcd.setCursor(-4,2);
    lcd.print(MSG_NOZZLE_TEMP);
    lcd.setCursor(7,2);
    lcd.print(itostr3(tHotend));
    lcd.print(LCD_STR_DEGREE);
    lcd.print('C');
    lcd.setCursor(4,3);
    lcd.print(LCD_STR_DOWNLEVEL[0]);
    lcd.setCursor(6,3);
    lcd.print(MSG_MOUNT);
}

static void lcd_nozzle_heating()
{
    MENU_PAGE = 401;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
    
    int thotend = int(degTargetHotend(0) + 0.5);
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_TEMP);
    lcd.setCursor(8, 1);
    lcd.print(LCD_STR_UPLEVEL[0]);
    lcd.setCursor(-4, 2);
    lcd.print(MSG_NOZZLE);
    lcd.setCursor(2, 2);
    lcd.print(itostr3(int(degHotend(0) + 0.5)));
    lcd.print('/');
    if (thotend == 0)
    {
        lcd.print(itostr3(NOZZLE_TEMP));
        target_temperature[0] = NOZZLE_TEMP;
        
        char cmd[30];
        sprintf_P(cmd, PSTR("M104 S%s"), itostr3left(NOZZLE_TEMP));
        enquecommand(cmd);
        HEAT_OFF = false;
    }
    else
    {
        //target_temperature[0] = thotend;
        lcd.print(itostr3(thotend));
    }
    
    lcd.print(LCD_STR_DEGREE);
    lcd.print('C');
    lcd.setCursor(4, 3);
    lcd.print(LCD_STR_DOWNLEVEL[0]);
    
    /*
    if (thotend != thotend_bak)
    {
        char cmd[30];
        char* c;
        sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(degTargetHotend(0) + 0.5)));
        enquecommand(cmd);
        
        thotend_bak = thotend;
    }
    */
}

static void lcd_bed_heating()
{
    MENU_PAGE = 402;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
    
    int tbed = int(degTargetBed() + 0.5);
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_TEMP);
    lcd.setCursor(8, 1);
    lcd.print(LCD_STR_UPLEVEL[0]);
    lcd.setCursor(-4, 2);
    lcd.print(MSG_BED);
    lcd.setCursor(2, 2);
    lcd.print(itostr3(int(degBed() + 0.5)));
    lcd.print('/');
    if (target_temperature_bed == 0)
    {
        lcd.print(itostr3(BED_TEMP));
        target_temperature_bed = BED_TEMP;
        
        char cmd[30];
        sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(BED_TEMP));
        enquecommand(cmd);
        BED_OFF = false;
    }
    else
    {
        lcd.print(itostr3(tbed));
    }
    
    lcd.print(LCD_STR_DEGREE);
    lcd.print('C');
    lcd.setCursor(4, 3);
    lcd.print(LCD_STR_DOWNLEVEL[0]);
}

static void lcd_stop_heat()
{
    enquecommand_P(PSTR("M104 S0"));
    enquecommand_P(PSTR("M140 S0"));
    
    HEAT_OFF = true;
    BED_OFF = true;
}

///



#ifdef SDSUPPORT
static void lcd_autostart_sd()
{
    card.lastnr=0;
    card.setroot();
    card.checkautostart(true);
}
#endif

/*
void lcd_preheat_pla()
{
    setTargetHotend0(plaPreheatHotendTemp);
    setTargetHotend1(plaPreheatHotendTemp);
    setTargetHotend2(plaPreheatHotendTemp);
    setTargetBed(plaPreheatHPBTemp);
    fanSpeed = plaPreheatFanSpeed;
    lcd_return_to_status();
}

void lcd_preheat_abs()
{
    setTargetHotend0(absPreheatHotendTemp);
    setTargetHotend1(absPreheatHotendTemp);
    setTargetHotend2(absPreheatHotendTemp);
    setTargetBed(absPreheatHPBTemp);
    fanSpeed = absPreheatFanSpeed;
    lcd_return_to_status();
}
*/

/*
static void lcd_tune_menu()
{
    //2014/04/19
    //MENU_PAGE = 5;
	
    START_MENU();
    MENU_ITEM(back, MSG_MAIN, lcd_main_menu);
    MENU_ITEM_EDIT(int3, MSG_SPEED, &feedmultiply, 10, 999);
    MENU_ITEM_EDIT(int3, MSG_NOZZLE, &target_temperature[0], 0, HEATER_0_MAXTEMP - 15);
#if TEMP_SENSOR_1 != 0
    MENU_ITEM_EDIT(int3, MSG_NOZZLE1, &target_temperature[1], 0, HEATER_1_MAXTEMP - 15);
#endif
#if TEMP_SENSOR_2 != 0
    MENU_ITEM_EDIT(int3, MSG_NOZZLE2, &target_temperature[2], 0, HEATER_2_MAXTEMP - 15);
#endif
#if TEMP_SENSOR_BED != 0
    MENU_ITEM_EDIT(int3, MSG_BED, &target_temperature_bed, 0, BED_MAXTEMP - 15);
#endif
    MENU_ITEM_EDIT(int3, MSG_FAN_SPEED, &fanSpeed, 0, 255);
    MENU_ITEM_EDIT(int3, MSG_FLOW, &extrudemultiply, 10, 999);
#ifdef FILAMENTCHANGEENABLE
     MENU_ITEM(gcode, MSG_FILAMENTCHANGE, PSTR("M600"));
#endif
    END_MENU();
}
*/

//2014/08/25
static void lcd_temp_monitor()
{
    MENU_PAGE = 500;
    
    encoderPosition = 25;
    
    //SERIAL_PROTOCOLPGM("lcd_temp_monitor"); 
    //SERIAL_PROTOCOLLN((int)encoderPosition);
    
    //if (encoderPosition < 5)
    //    encoderPosition = 5;

    int tHotend=int(degHotend(0) + 0.5);
    int tTarget=int(degTargetHotend(0) + 0.5);
    
    lcd.setCursor(0, 0);
    lcd.print(MSG_TEMP_MON_TITLE);
    lcd.setCursor(0, 1);
    lcd.print(MSG_NOZZLE);
    lcd.setCursor(6, 1);
    lcd.print(itostr3(int(degHotend(0) + 0.5)));
    lcd.print('/');
    lcd.print(itostr3left(int(target_temperature[0])));
    //lcd.print(itostr3(NOZZLE_TEMP));
    lcd.print(LCD_STR_DEGREE);
    lcd.print('C');
    lcd.setCursor(-4, 2);
    lcd.print(MSG_BED);
    lcd.setCursor(2, 2);
    lcd.print(itostr3(int(degBed() + 0.5)));
    lcd.print('/');
    lcd.print(itostr3left(int(target_temperature_bed)));
    //lcd.print(itostr3(BED_TEMP));
    lcd.print(LCD_STR_DEGREE);
    lcd.print('C');
}
/// 2014/08/25


static void lcd_preheat_menu()
{
    MENU_PAGE = 400;

    if (encoderPosition < 5)
        encoderPosition = 5;
    
    if ((HEAT_OFF == true) && (BED_OFF == true))
    {
        //SERIAL_PROTOCOLLN("OFF/OFF");
        
        START_MENU();
        MENU_ITEM(function, MSG_PREHEAT_TITLE, lcd_main_menu);
        MENU_ITEM(submenu, MSG_NOZZLE_OFF, lcd_nozzle_heating);
        MENU_ITEM(submenu, MSG_BED_OFF, lcd_bed_heating);
        MENU_ITEM(function, MSG_HEAT_STOP, lcd_stop_heat);
        END_MENU();
    }
    if ((HEAT_OFF == false) && (BED_OFF == true))
    {
        //SERIAL_PROTOCOLLN("ON/OFF");
        
        START_MENU();
        MENU_ITEM(function, MSG_PREHEAT_TITLE, lcd_main_menu);
        MENU_ITEM(submenu, MSG_NOZZLE_ON, lcd_nozzle_heating);
        MENU_ITEM(submenu, MSG_BED_OFF, lcd_bed_heating);
        MENU_ITEM(function, MSG_HEAT_STOP, lcd_stop_heat);
        END_MENU();
    }
    if ((HEAT_OFF == true) && (BED_OFF == false))
    {
        //SERIAL_PROTOCOLLN("OFF/ON");
        
        START_MENU();
        MENU_ITEM(function, MSG_PREHEAT_TITLE, lcd_main_menu);
        MENU_ITEM(submenu, MSG_NOZZLE_OFF, lcd_nozzle_heating);
        MENU_ITEM(submenu, MSG_BED_ON, lcd_bed_heating);
        MENU_ITEM(function, MSG_HEAT_STOP, lcd_stop_heat);
        END_MENU();
    }
    if ((HEAT_OFF == false ) && (BED_OFF == false))
    {
        //SERIAL_PROTOCOLLN("ON/ON");
        
        START_MENU();
        MENU_ITEM(function, MSG_PREHEAT_TITLE, lcd_main_menu);
        MENU_ITEM(submenu, MSG_NOZZLE_ON, lcd_nozzle_heating);
        MENU_ITEM(submenu, MSG_BED_ON, lcd_bed_heating);
        MENU_ITEM(function, MSG_HEAT_STOP, lcd_stop_heat);
        END_MENU();
	}
    lcd.setCursor(15,1);
    lcd.print(' ');
    lcd.setCursor(11,2);
    lcd.print(' ');
}


float move_menu_scale;

static void lcd_move_menu_axis();


/*
static void lcd_move_x()
{
    //2014/04/19
    //int ret;
    
    //if (MENU_PAGE == 210) ret = 210;
    //else ret = 211;
    
    MENU_PAGE = 350;
    
    if (encoderPosition != 0)
    {
        current_position[X_AXIS] += float((int)encoderPosition) * move_menu_scale;
        if (current_position[X_AXIS] < X_MIN_POS)
            current_position[X_AXIS] = X_MIN_POS;
        if (current_position[X_AXIS] > X_MAX_POS)
            current_position[X_AXIS] = X_MAX_POS;
        encoderPosition = 0;
        plan_buffer_line(current_position[X_AXIS], current_position[Y_AXIS], current_position[Z_AXIS], current_position[E_AXIS], 600, active_extruder);
        lcdDrawUpdate = 1;
    }
    if (lcdDrawUpdate)
    {
        lcd_implementation_drawedit(PSTR("X"), ftostr31(current_position[X_AXIS]));
    }
    if (LCD_CLICKED)
    {
        //2014/04/19
        MENU_PAGE = 301; //ret;
        
        lcd_quick_feedback();
        currentMenu = lcd_move_menu_axis;
        encoderPosition = 0;
    }
}
*/
/*
static void lcd_move_y()
{
    //2014/04/19
    int ret;
    
    if (MENU_PAGE == 210) ret = 210;
    else ret = 211;
    MENU_PAGE = 250;
    
    if (encoderPosition != 0)
    {
        current_position[Y_AXIS] += float((int)encoderPosition) * move_menu_scale;
        if (current_position[Y_AXIS] < Y_MIN_POS)
            current_position[Y_AXIS] = Y_MIN_POS;
        if (current_position[Y_AXIS] > Y_MAX_POS)
            current_position[Y_AXIS] = Y_MAX_POS;
        encoderPosition = 0;
        plan_buffer_line(current_position[X_AXIS], current_position[Y_AXIS], current_position[Z_AXIS], current_position[E_AXIS], 600, active_extruder);
        lcdDrawUpdate = 1;
    }
    if (lcdDrawUpdate)
    {
        lcd_implementation_drawedit(PSTR("Y"), ftostr31(current_position[Y_AXIS]));
    }
    if (LCD_CLICKED)
    {
        //2014/04/19
        MENU_PAGE = ret;
        
        lcd_quick_feedback();
        currentMenu = lcd_move_menu_axis;
        encoderPosition = 0;
    }
}
*/

/*
static void lcd_move_z()
{
    //2014/04/19
    int ret;
    
    if (MENU_PAGE == 210) ret = 210;
    else ret = 211;
    MENU_PAGE = 250;
    
    if (encoderPosition != 0)
    {
        current_position[Z_AXIS] += float((int)encoderPosition) * move_menu_scale;
        if (current_position[Z_AXIS] < Z_MIN_POS)
            current_position[Z_AXIS] = Z_MIN_POS;
        if (current_position[Z_AXIS] > Z_MAX_POS)
            current_position[Z_AXIS] = Z_MAX_POS;
        encoderPosition = 0;
        plan_buffer_line(current_position[X_AXIS], current_position[Y_AXIS], current_position[Z_AXIS], current_position[E_AXIS], 60, active_extruder);
        lcdDrawUpdate = 1;
    }
    if (lcdDrawUpdate)
    {
        lcd_implementation_drawedit(PSTR("Z"), ftostr31(current_position[Z_AXIS]));
    }
    if (LCD_CLICKED)
    {
        //2014/04/19
        MENU_PAGE = ret;
        
        lcd_quick_feedback();
        currentMenu = lcd_move_menu_axis;
        encoderPosition = 0;
    }
}
*/

/*
static void lcd_move_e()
{
    //2014/04/19
    int ret;
    
    if (MENU_PAGE == 210) ret = 210;
    else ret = 211;
    MENU_PAGE = 250;
        
    if (encoderPosition != 0)
    {
        current_position[E_AXIS] += float((int)encoderPosition) * move_menu_scale;
        encoderPosition = 0;
        plan_buffer_line(current_position[X_AXIS], current_position[Y_AXIS], current_position[Z_AXIS], current_position[E_AXIS], 20, active_extruder);
        lcdDrawUpdate = 1;
    }
    if (lcdDrawUpdate)
    {
        lcd_implementation_drawedit(PSTR("Extruder"), ftostr31(current_position[E_AXIS]));
    }
    if (LCD_CLICKED)
    {
        //2014/04/19
        MENU_PAGE = ret;
        
        lcd_quick_feedback();
        currentMenu = lcd_move_menu_axis;
        encoderPosition = 0;
    }
}
*/

/*
static void lcd_move_menu_axis()
{
    START_MENU();
    MENU_ITEM(function, MSG_MOVE_AXIS, lcd_move_menu);
    MENU_ITEM(submenu, "X AXIS 10mm", lcd_move_x);
    MENU_ITEM(submenu, "Y AXIS 10mm", lcd_move_y);
    MENU_ITEM(submenu, "Z AXIS 10mm", lcd_move_z);
    END_MENU();
}
*/

/*
static void lcd_move_menu_10mm()
{
    //2014/04/19
    MENU_PAGE = 301;
    
    move_menu_scale = 10.0;
    lcd_move_menu_axis();
}
*/

/*
static void lcd_move_menu_1mm()
{
    //2014/04/19
    MENU_PAGE = 301;
    
    move_menu_scale = 1.0;
    lcd_move_menu_axis();
}
*/
/*
static void lcd_move_menu_01mm()
{
    //2014/04/19
    MENU_PAGE = 302;
    
    move_menu_scale = 0.1;
    lcd_move_menu_axis();
}
*/

/*
static void lcd_move_menu()
{
    //2014/04/19
    //MENU_PAGE = 21;
	
    START_MENU();
    //MENU_ITEM(function, MSG_PREPARE, lcd_prepare_menu);
    MENU_ITEM(submenu, "Move 10mm", lcd_move_menu_10mm);
    MENU_ITEM(submenu, "Move 1mm", lcd_move_menu_1mm);
    MENU_ITEM(submenu, "Move 0.1mm", lcd_move_menu_01mm);
    //TODO:X,Y,Z,E
    END_MENU();
}
*/

static void lcd_move_x_menu()
{
    //2014/04/19
    MENU_PAGE = 300;
/*
    if (encoderPosition < 5)
        encoderPosition = 5;

    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
*/
    lcd.setCursor(0, 0);
    lcd.print(MSG_BUTTON);
    lcd.setCursor(0, 1);
    lcd.print(MSG_XUP);
    lcd.setCursor(-4, 2);
    lcd.print(LCD_STR_LEFTARROW[0]);
    lcd.print("Z   \x28\xd2\xc6\xad\xb0\x29   Y");  //<Z  (ҎƎ)   Y>
    lcd.print(LCD_STR_RIGHTARROW[0]);
    lcd.setCursor(-4, 3);
    lcd.print(MSG_XDOWN);
}

static void lcd_move_y_menu()
{
    MENU_PAGE = 301;

/*
    if (encoderPosition < 5)
        encoderPosition = 5;

    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
*/    
    lcd.setCursor(0, 0);
    lcd.print(MSG_BUTTON);
    lcd.setCursor(0, 1);
    lcd.print(MSG_YUP);
    lcd.setCursor(-4, 2);
    lcd.print(LCD_STR_LEFTARROW[0]);
    lcd.print("X   \x28\xd2\xc6\xad\xb0\x29    ");  //<X  (ҎƎ)   >
    lcd.print(LCD_STR_RIGHTARROW[0]);
    lcd.setCursor(-4, 3);
    lcd.print(MSG_YDOWN);
}

static void lcd_move_z_menu()
{
    MENU_PAGE = 302;

/*
    if (encoderPosition < 5)
        encoderPosition = 5;

    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }
*/

    lcd.setCursor(0, 0);
    lcd.print(MSG_BUTTON);
    lcd.setCursor(0, 1);
    lcd.print(MSG_ZDOWN);
    lcd.setCursor(-4, 2);
    //lcd.print("<X   \x28\xd2\xc6\xad\xb0\x29    >");  //<X  (ҎƎ)   >
    lcd.print(LCD_STR_LEFTARROW[0]);
    lcd.print("    \x28\xd2\xc6\xad\xb0\x29   X");  //<  (ҎƎ)   X>
    lcd.print(LCD_STR_RIGHTARROW[0]);
    lcd.setCursor(-4, 3);
    lcd.print(MSG_ZUP);
}

static void lcd_xmove50()
{
    MENU_PAGE = 303;
    
    if (encoderPosition < 5)
        encoderPosition = 5;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }

    if (t_xmove != 3)
    {
        lcd.setCursor(0, 0);
        lcd.print(MSG_MOVE_XYZ);
        lcd.setCursor(1, 1);
        lcd.print("50mm             ");
        lcd.setCursor(-3, 2);
        lcd.print("10mm             ");
        lcd.setCursor(-3, 3);
        lcd.print(" 1mm             ");
        
        if (t_xmove == 0)
        {
            encoderPosition = 5;
            lcd.setCursor(-4, 2);
            lcd.print(' ');
            lcd.setCursor(-4, 3);
            lcd.print(' ');
            lcd.setCursor(0, 1);
            lcd.print('>');
        }
        if (t_xmove == 1)
        {
            encoderPosition == 10;
            lcd.setCursor(0, 1);
            lcd.print(' ');
            lcd.setCursor(-4, 3);
            lcd.print(' ');
            lcd.setCursor(-4, 2);
            lcd.print('>');
        }
        if (t_xmove == 2)
        {
            encoderPosition == 15;
            lcd.setCursor(0, 1);
            lcd.print(' ');
            lcd.setCursor(-4, 2);
            lcd.print(' ');
            lcd.setCursor(-4, 3);
            lcd.print('>');
        }
    }
    else
    {
        //t_xmove = 3;
        encoderPosition == 15;
        lcd.setCursor(0, 0);
        lcd.print(MSG_MOVE_XYZ);
        lcd.setCursor(1, 1);
        lcd.print("10mm             ");
        lcd.setCursor(-3, 2);
        lcd.print(" 1mm             ");
        lcd.setCursor(-3, 3);
        lcd.print(MSG_HOME);
        lcd.setCursor(-4, 3);
        lcd.print('>');
    }
}

static void lcd_ymove50()
{
    MENU_PAGE = 304;
    
    if (encoderPosition < 5)
        encoderPosition = 5;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }

    if (t_ymove != 3)
    {
        lcd.setCursor(0, 0);
        lcd.print(MSG_MOVE_XYZ);
        lcd.setCursor(1, 1);
        lcd.print("50mm             ");
        lcd.setCursor(-3, 2);
        lcd.print("10mm             ");
        lcd.setCursor(-3, 3);
        lcd.print(" 1mm             ");
        
        if (t_ymove == 0)
        {
            encoderPosition = 5;
            lcd.setCursor(-4, 2);
            lcd.print(' ');
            lcd.setCursor(-4, 3);
            lcd.print(' ');
            lcd.setCursor(0, 1);
            lcd.print('>');
        }
        if (t_ymove == 1)
        {
            encoderPosition = 10;
            lcd.setCursor(0, 1);
            lcd.print(' ');
            lcd.setCursor(-4, 3);
            lcd.print(' ');
            lcd.setCursor(-4, 2);
            lcd.print('>');
        }
        if (t_ymove == 2)
        {
            encoderPosition = 15;
            lcd.setCursor(0, 1);
            lcd.print(' ');
            lcd.setCursor(-4, 2);
            lcd.print(' ');
            lcd.setCursor(-4, 3);
            lcd.print('>');
        }
    }
    else
    {
        //t_ymove = 3;
        encoderPosition = 20;
        lcd.setCursor(0, 0);
        lcd.print(MSG_MOVE_XYZ);
        lcd.setCursor(1, 1);
        lcd.print("10mm             ");
        lcd.setCursor(-3, 2);
        lcd.print(" 1mm             ");
        lcd.setCursor(-3, 3);
        lcd.print(MSG_HOME);
        lcd.setCursor(-4, 3);
        lcd.print('>');
    }
}

static void lcd_zmove50()
{
    MENU_PAGE = 305;
    
    if (encoderPosition < 5)
        encoderPosition = 5;
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
    }

    if (t_zmove != 3)
    {
        lcd.setCursor(0, 0);
        lcd.print(MSG_MOVE_XYZ);
        lcd.setCursor(1, 1);
        lcd.print("50mm             ");
        lcd.setCursor(-3, 2);
        lcd.print("10mm             ");
        lcd.setCursor(-3, 3);
        lcd.print(" 1mm             ");
        
        if (t_zmove == 0)
        {
            encoderPosition = 5;
            lcd.setCursor(-4, 2);
            lcd.print(' ');
            lcd.setCursor(-4, 3);
            lcd.print(' ');
            lcd.setCursor(0, 1);
            lcd.print('>');
        }
        if (t_zmove == 1)
        {
            encoderPosition = 10;
            lcd.setCursor(0, 1);
            lcd.print(' ');
            lcd.setCursor(-4, 3);
            lcd.print(' ');
            lcd.setCursor(-4, 2);
            lcd.print('>');
        }
        if (t_zmove == 2)
        {
            encoderPosition = 15;
            lcd.setCursor(0, 1);
            lcd.print(' ');
            lcd.setCursor(-4, 2);
            lcd.print(' ');
            lcd.setCursor(-4, 3);
            lcd.print('>');
        }
    }
    else
    {
        //t_zmove = 3;
        encoderPosition = 20;
        lcd.setCursor(0, 0);
        lcd.print(MSG_MOVE_XYZ);
        lcd.setCursor(1, 1);
        lcd.print("10mm             ");
        lcd.setCursor(-3, 2);
        lcd.print(" 1mm             ");
        lcd.setCursor(-3, 3);
        lcd.print(MSG_HOME);
        lcd.setCursor(-4, 3);
        lcd.print('>');
    }
}

/*
static void lcd_control_temperature_menu()
{
    //2014/04/19
    //MENU_PAGE = 31;
	
	// set up temp variables - undo the default scaling
	raw_Ki = unscalePID_i(Ki);
	raw_Kd = unscalePID_d(Kd);
	
    START_MENU();
    MENU_ITEM(back, MSG_CONTROL, lcd_control_menu);
    MENU_ITEM_EDIT(int3, MSG_NOZZLE, &target_temperature[0], 0, HEATER_0_MAXTEMP - 15);
#if TEMP_SENSOR_1 != 0
    MENU_ITEM_EDIT(int3, MSG_NOZZLE1, &target_temperature[1], 0, HEATER_1_MAXTEMP - 15);
#endif
#if TEMP_SENSOR_2 != 0
    MENU_ITEM_EDIT(int3, MSG_NOZZLE2, &target_temperature[2], 0, HEATER_2_MAXTEMP - 15);
#endif
#if TEMP_SENSOR_BED != 0
    MENU_ITEM_EDIT(int3, MSG_BED, &target_temperature_bed, 0, BED_MAXTEMP - 15);
#endif
    MENU_ITEM_EDIT(int3, MSG_FAN_SPEED, &fanSpeed, 0, 255);
#ifdef AUTOTEMP
    MENU_ITEM_EDIT(bool, MSG_AUTOTEMP, &autotemp_enabled);
    MENU_ITEM_EDIT(float3, MSG_MIN, &autotemp_min, 0, HEATER_0_MAXTEMP - 15);
    MENU_ITEM_EDIT(float3, MSG_MAX, &autotemp_max, 0, HEATER_0_MAXTEMP - 15);
    MENU_ITEM_EDIT(float32, MSG_FACTOR, &autotemp_factor, 0.0, 1.0);
#endif
#ifdef PIDTEMP
    MENU_ITEM_EDIT(float52, MSG_PID_P, &Kp, 1, 9990);
	// i is typically a small value so allows values below 1
    MENU_ITEM_EDIT_CALLBACK(float52, MSG_PID_I, &raw_Ki, 0.01, 9990, copy_and_scalePID_i);
    MENU_ITEM_EDIT_CALLBACK(float52, MSG_PID_D, &raw_Kd, 1, 9990, copy_and_scalePID_d);
# ifdef PID_ADD_EXTRUSION_RATE
    MENU_ITEM_EDIT(float3, MSG_PID_C, &Kc, 1, 9990);
# endif//PID_ADD_EXTRUSION_RATE
#endif//PIDTEMP
    MENU_ITEM(submenu, MSG_PREHEAT_PLA_SETTINGS, lcd_control_temperature_preheat_pla_settings_menu);
    MENU_ITEM(submenu, MSG_PREHEAT_ABS_SETTINGS, lcd_control_temperature_preheat_abs_settings_menu);
    END_MENU();
}
*/
/*
static void lcd_control_temperature_preheat_pla_settings_menu()
{
    START_MENU();
    MENU_ITEM(back, MSG_TEMPERATURE, lcd_control_temperature_menu);
    MENU_ITEM_EDIT(int3, MSG_FAN_SPEED, &plaPreheatFanSpeed, 0, 255);
    MENU_ITEM_EDIT(int3, MSG_NOZZLE, &plaPreheatHotendTemp, 0, HEATER_0_MAXTEMP - 15);
#if TEMP_SENSOR_BED != 0
    MENU_ITEM_EDIT(int3, MSG_BED, &plaPreheatHPBTemp, 0, BED_MAXTEMP - 15);
#endif
#ifdef EEPROM_SETTINGS
    MENU_ITEM(function, MSG_STORE_EPROM, Config_StoreSettings);
#endif
    END_MENU();
}
*/
/*
static void lcd_control_temperature_preheat_abs_settings_menu()
{
	
    START_MENU();
    MENU_ITEM(back, MSG_TEMPERATURE, lcd_control_temperature_menu);
    MENU_ITEM_EDIT(int3, MSG_FAN_SPEED, &absPreheatFanSpeed, 0, 255);
    MENU_ITEM_EDIT(int3, MSG_NOZZLE, &absPreheatHotendTemp, 0, HEATER_0_MAXTEMP - 15);
#if TEMP_SENSOR_BED != 0
    MENU_ITEM_EDIT(int3, MSG_BED, &absPreheatHPBTemp, 0, BED_MAXTEMP - 15);
#endif
#ifdef EEPROM_SETTINGS
    MENU_ITEM(function, MSG_STORE_EPROM, Config_StoreSettings);
#endif
    END_MENU();
}
*/
/*
static void lcd_control_motion_menu()
{
    //2014/04/19
    //MENU_PAGE = 400;
	
    START_MENU();
    MENU_ITEM(back, MSG_CONTROL, lcd_control_menu);
    MENU_ITEM_EDIT(float5, MSG_ACC, &acceleration, 500, 99000);
    MENU_ITEM_EDIT(float3, MSG_VXY_JERK, &max_xy_jerk, 1, 990);
    MENU_ITEM_EDIT(float52, MSG_VZ_JERK, &max_z_jerk, 0.1, 990);
    MENU_ITEM_EDIT(float3, MSG_VE_JERK, &max_e_jerk, 1, 990);
    MENU_ITEM_EDIT(float3, MSG_VMAX MSG_X, &max_feedrate[X_AXIS], 1, 999);
    MENU_ITEM_EDIT(float3, MSG_VMAX MSG_Y, &max_feedrate[Y_AXIS], 1, 999);
    MENU_ITEM_EDIT(float3, MSG_VMAX MSG_Z, &max_feedrate[Z_AXIS], 1, 999);
    MENU_ITEM_EDIT(float3, MSG_VMAX MSG_E, &max_feedrate[E_AXIS], 1, 999);
    MENU_ITEM_EDIT(float3, MSG_VMIN, &minimumfeedrate, 0, 999);
    MENU_ITEM_EDIT(float3, MSG_VTRAV_MIN, &mintravelfeedrate, 0, 999);
    MENU_ITEM_EDIT_CALLBACK(long5, MSG_AMAX MSG_X, &max_acceleration_units_per_sq_second[X_AXIS], 100, 99000, reset_acceleration_rates);
    MENU_ITEM_EDIT_CALLBACK(long5, MSG_AMAX MSG_Y, &max_acceleration_units_per_sq_second[Y_AXIS], 100, 99000, reset_acceleration_rates);
    MENU_ITEM_EDIT_CALLBACK(long5, MSG_AMAX MSG_Z, &max_acceleration_units_per_sq_second[Z_AXIS], 100, 99000, reset_acceleration_rates);
    MENU_ITEM_EDIT_CALLBACK(long5, MSG_AMAX MSG_E, &max_acceleration_units_per_sq_second[E_AXIS], 100, 99000, reset_acceleration_rates);
    MENU_ITEM_EDIT(float5, MSG_A_RETRACT, &retract_acceleration, 100, 99000);
    MENU_ITEM_EDIT(float52, MSG_XSTEPS, &axis_steps_per_unit[X_AXIS], 5, 9999);
    MENU_ITEM_EDIT(float52, MSG_YSTEPS, &axis_steps_per_unit[Y_AXIS], 5, 9999);
    MENU_ITEM_EDIT(float51, MSG_ZSTEPS, &axis_steps_per_unit[Z_AXIS], 5, 9999);
    MENU_ITEM_EDIT(float51, MSG_ESTEPS, &axis_steps_per_unit[E_AXIS], 5, 9999);    
#ifdef ABORT_ON_ENDSTOP_HIT_FEATURE_ENABLED
    MENU_ITEM_EDIT(bool, "Endstop abort", &abort_on_endstop_hit);
#endif
    END_MENU();
}
*/

/*
#ifdef FWRETRACT
static void lcd_control_retract_menu()
{
    START_MENU();
    MENU_ITEM(back, MSG_CONTROL, lcd_control_menu);
    MENU_ITEM_EDIT(bool, MSG_AUTORETRACT, &autoretract_enabled);
    MENU_ITEM_EDIT(float52, MSG_CONTROL_RETRACT, &retract_length, 0, 100);
    MENU_ITEM_EDIT(float3, MSG_CONTROL_RETRACTF, &retract_feedrate, 1, 999);
    MENU_ITEM_EDIT(float52, MSG_CONTROL_RETRACT_ZLIFT, &retract_zlift, 0, 999);
    MENU_ITEM_EDIT(float52, MSG_CONTROL_RETRACT_RECOVER, &retract_recover_length, 0, 100);
    MENU_ITEM_EDIT(float3, MSG_CONTROL_RETRACT_RECOVERF, &retract_recover_feedrate, 1, 999);
    END_MENU();
}
#endif
*/

#if SDCARDDETECT == -1
static void lcd_sd_refresh()
{
    card.initsd();
    currentMenuViewOffset = 0;
}
#endif
static void lcd_sd_updir()
{
    card.updir();
    currentMenuViewOffset = 0;
}

/*
static void lcd_no_action()
{
    return;
}
*/


static int RC = 0;

void lcd_sdcard_menu()
{
    uint16_t fileCnt;
    
    MENU_PAGE = 100;
    
    if (READ(SDCARDDETECT) != 0)
    {
        MENU_PAGE = 101;
        
        card.cardOK == false;
        sd_detect = false;
        
        if (no_card_flg == false)
        {
            lcd_cls();
            //SERIAL_PROTOCOLLN("NO CARD FLG");
            lcd.setCursor(0, 0);
            lcd.print(MSG_PRINT_TITLE);
            lcd.setCursor(0, 1);
            lcd.print(MSG_NO_CARD);
            no_card_flg = true;
            SD_FILE_COUNT = 0;
            sd_file_pos_bak = -1;
            Fcnt = 0;
        }
        //SERIAL_PROTOCOLLN("sd_file_pos == sd_file_pos_bak return 0");
        //SERIAL_PROTOCOLPGM("no_card_flg = ");
        //SERIAL_PROTOCOLLN(no_card_flg);
        return;
    }
    else if (sd_file_pos == sd_file_pos_bak)
    {
        if (RC != 2)
        {
            RC++;
        }
        else
        {
            //SERIAL_PROTOCOLLN("sd_file_pos == sd_file_pos_bak return 1");
            //RC = 0;
            return;
        }
    }
    no_card_flg = false;
    
    if (encoderPosition < 5)
        encoderPosition = 5;
    
    //2014/07/18
    //SERIAL_PROTOCOLLN("CARD CHECK");
    
    if (lcd_cls_flg == false)
    {
        lcd_cls_flg = true;
        lcd_cls();
        //lcd.setCursor(0, 0);
        //lcd.print(MSG_PRINT_TITLE);
        //SERIAL_PROTOCOLLN("SD CLS");
    }
    
    if (READ(SDCARDDETECT) != 0)
    {
        MENU_PAGE = 101;
        
        sd_detect = false;
        card.cardOK = false;
        
        //SERIAL_PROTOCOLLN("NO CARD");
        lcd.setCursor(0, 0);
        lcd.print(MSG_PRINT_TITLE);
        lcd.setCursor(0, 1);
        lcd.print(MSG_NO_CARD);
        enquecommand_P(PSTR("M22"));
    }
    else
    {
        //MENU_PAGE = 100;
        
        sd_print_flg = false;
        sd_detect = true;
        //SERIAL_PROTOCOLLN("CARD DETECTED");
        
        if (card.cardOK == false)
        {
            MENU_PAGE = 101;
            
            //SERIAL_PROTOCOLLN("init. SD");
            //2014/08/25
            lcd.setCursor(0, 0);
            lcd.print(MSG_PRINT_TITLE);
            lcd.setCursor(0, 1);
            lcd.print(MSG_NO_CARD);
            //enquecommand_P(PSTR("M22"));
            
            ///
            
            for (int i = 0; i < 20; i++)
            {
                //enquecommand_P(PSTR("M22"));
                card.initsd();  //init sd card
                
                //SERIAL_PROTOCOLLN("card check1");
                
                if (card.cardOK == true)
                {
                    MENU_PAGE = 100;
                    //SERIAL_PROTOCOLLN("cardOK1");
                    //delay(00);
                    break;
                }
            }
            if (card.cardOK == false)
            {
                MENU_PAGE = 101;
                return;
            }
            fileCnt = card.getnrfilenames();  //file(xxx.gcode) +folder count
        }
        else
        {
            char dir[100];
            dir[0] = *card.getWorkDirName();
            
            if(dir[0] !='/')
            {
                //lcd.setCursor(0,0);
                //lcd.print(MSG_PRINT_TITLE);
                
                //SERIAL_PROTOCOLPGM("SD:Workdir = ");
                //SERIAL_PROTOCOLLN(card.getWorkDirName());
            }
            else
            {
                for (int i = 0; i < 20; i++)
                {
                    //enquecommand_P(PSTR("M22"));
                    card.initsd();  //init sd card
                    
                    //SERIAL_PROTOCOLLN("card check2");
                    
                    if (card.cardOK == true)
                    {
                        //SERIAL_PROTOCOLLN("cardOK2");
                        //delay(00);
                        break;
                    }
                }
                if (card.cardOK == false)
                {
                    MENU_PAGE = 101;
                    
                    //enquecommand_P(PSTR("M22"));
                    return;
                }
                //SERIAL_PROTOCOLLN("cardOK2 delay2000");
                //delay(2000);
            }
            fileCnt = card.getnrfilenames();  //file(xxx.gcode) +folder count
        }
        
        SD_FILE_COUNT = fileCnt;
        Fcnt = fileCnt;
        
        //SERIAL_PROTOCOLPGM("Fcnt = ");
        //SERIAL_PROTOCOLLN(Fcnt);
        
        /*
        for (int xx = 0; xx < fileCnt; xx++)
        {
             card.getfilename(xx + sd_curpos);
             if (card.filenameIsDir)
             {
                 SERIAL_PROTOCOLPGM("folder = ");
                 SERIAL_PROTOCOL(card.filename);
                 SERIAL_PROTOCOLPGM(" , ");
                 SERIAL_PROTOCOL(card.longFilename);
                 SERIAL_PROTOCOLPGM(" , [0] = ");
                 SERIAL_PROTOCOLLN((int)card.longFilename[0]);
                 SERIAL_PROTOCOLLN((int)sizeof(card.longFilename));
             }
             else
             {
                 SERIAL_PROTOCOLPGM("file = ");
                 SERIAL_PROTOCOL(card.filename);
                 SERIAL_PROTOCOLPGM(" , ");
                 SERIAL_PROTOCOLLN(card.longFilename);
             }
        }
        */
        
        
        lcd.setCursor(0,0);
        lcd.print(MSG_PRINT_TITLE);
        
        card.getWorkDirName();
        //SERIAL_PROTOCOLPGM("SD:Workdir2 = ");
        //SERIAL_PROTOCOLLN(card.getWorkDirName());
        
        if(card.filename[0]=='/')
        {
#if SDCARDDETECT == -1
            //MENU_ITEM(function, LCD_STR_REFRESH MSG_REFRESH, lcd_sd_refresh);
#endif
        }
        else
        {
//            SD_FILE_COUNT += 1;
            //MENU_ITEM(function, LCD_STR_FOLDER "..", lcd_sd_updir);
            //SERIAL_PROTOCOLPGM("FOLDER");
            //SERIAL_PROTOCOLLN("");
        }
        
        //SERIAL_PROTOCOLPGM("SD::sd_file_pos = ");
        //SERIAL_PROTOCOLLN(sd_file_pos);
        
        if (sd_file_pos > 2)
        {
            sd_curpos = sd_file_pos - 2;
        }
        else
        {
            sd_curpos = 0;//sd_file_pos;
        }
        //SERIAL_PROTOCOLPGM("cursor_pos = ");
        //SERIAL_PROTOCOLLN(sd_curpos);
        
        
        for(uint16_t i = 0; i < 3; i++)
        {
            if (SD_FILE_COUNT == 0)
            {
                //lcd.setCursor(0,0);
                //lcd.print(MSG_PRINT_TITLE);
                //sd_file_pos_bak = 0;
                //sd_file_pos = 0;
                //SERIAL_PROTOCOLPGM("sd_file_pos_bak = ");
                //SERIAL_PROTOCOLLN(sd_file_pos_bak);
                break;
            }    
            if ( i >= SD_FILE_COUNT)
            {
                break;
            }
            //if (_menuItemNr == _lineNr)
            {
                card.getfilename(i + sd_curpos);
                //SERIAL_PROTOCOLLN(card.longFilename);
                
                if (card.filenameIsDir)
                {
                    //MENU_ITEM(sddirectory, MSG_CARD_MENU, card.filename, card.longFilename);
                    if ( i > 0)
                    {
                        lcd.setCursor(-4, i + 1);
                        lcd.print("                ");
                        lcd.setCursor(-3, i + 1);
                    }
                    else
                    {
                        lcd.setCursor(0, i + 1);
                        lcd.print("                ");
                        lcd.setCursor(1, i + 1);
                    }
                    
                    //normal, longFileȽ
                    char fn[16];

                    //if (card.longFilename[0] == 0)
                    {
                        //normal name  8ʸʲ
                        for(int j = 0; j < 9; j++)
                        {
                            if (card.filename[j] == 0x7e)
                                fn[j+1] = LCD_STR_TILDE[0];
                            else
                                fn[j+1] = card.filename[j];
                        }
                        fn[0] = LCD_STR_FOLDER[0];
                        //SERIAL_PROTOCOLPGM("folder = ");
                        //SERIAL_PROTOCOLLN(fn);
                    }
                    /*
                    else
                    {
                        //9ʸʾ
                        /*
                        for(int j = 0; j < 15; j++)
                        {
                            if (card.longFilename[j] == 0x7e)
                                fn[j+1] = LCD_STR_TILDE[0];
                            else
                                fn[j+1] = card.longFilename[j];
                            
                            if (card.longFilename[j] == 0)
                            {
                                SERIAL_PROTOCOLPGM("folder9 j < 15 = ");
                                SERIAL_PROTOCOLLN(j);
                                break;
                            }
                            else
                            {
                            }
                        }
                        //fn[15] = '\0';
                        fn[0] = LCD_STR_FOLDER[0];
                        SERIAL_PROTOCOLPGM("folder9 = ");
                        SERIAL_PROTOCOLLN(fn);
                    }
                    fn[0] = LCD_STR_FOLDER[0];
                    */
                    
                    lcd.print(fn/*card.filename*/);
                    
                    if (sd_file_pos == 0) 
                    {
                        lcd.setCursor( 0, 1);
                        lcd.print('>');
                    }
                    else if (sd_file_pos == 1) 
                    {
                        lcd.setCursor(-4, 2);
                        lcd.print('>');
                    }
                    else
                    {
                        lcd.setCursor(-4, 3);
                        lcd.print('>');
                    }
                    
                }
                else 
                {
                    //MENU_ITEM(sdfile, MSG_CARD_MENU, card.filename, card.longFilename);
                    
                    if ( i > 0) 
                    {
                        lcd.setCursor(-4, i + 1);
                        lcd.print("                ");
                        lcd.setCursor(-3, i + 1);
                    }
                    else
                    {
                        lcd.setCursor(0, 1);
                        lcd.print("                ");
                        lcd.setCursor(1, 1);
                    }
                    
                    char fn[13] = { 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0 };

                    for(int j = 0; j < 13; j++)
                    {
                        if (card.filename[j] == 0x7e)
                            fn[j] = LCD_STR_TILDE[0];
                        else
                            fn[j] = card.filename[j];
                    }
                    
                    //lcd.print(card.filename);
                    lcd.print(fn);
                    
                    //
                    //for(int j = 0; j < 14; j++)
                    //    lcd.print(card.longFilename[j]);  ///*card.longFilename*/);
                    //lcd.print(LCD_STR_TILDE[0]);
                    
                    if (sd_file_pos == 0) 
                    {
                        lcd.setCursor( 0, 1);
                        lcd.print('>');
                    }
                    else if (sd_file_pos == 1) 
                    {
                        lcd.setCursor(-4, 2);
                        lcd.print('>');
                    }
                    else
                    {
                        lcd.setCursor(-4, 3);
                        lcd.print('>');
                    }
                    
                    //SERIAL_PROTOCOLPGM("file = ");
                    //SERIAL_PROTOCOLLN(fn);
                }
            } 
            sd_file_pos_bak = sd_file_pos;
        }
        
        //lcd.setCursor(0,0);
        //lcd.print(MSG_PRINT_TITLE);
        
        //SERIAL_PROTOCOLLN("");
        //SERIAL_PROTOCOLPGM("SD FILE COUNT = ");
        //SERIAL_PROTOCOLLN(SD_FILE_COUNT);
        //SERIAL_PROTOCOLPGM("SD FCnt = ");
        //SERIAL_PROTOCOLLN(Fcnt);
    }
}

/*
#define menu_edit_type(_type, _name, _strFunc, scale) \
    void menu_edit_ ## _name () \
    { \
        if ((int32_t)encoderPosition < minEditValue) \
            encoderPosition = minEditValue; \
        if ((int32_t)encoderPosition > maxEditValue) \
            encoderPosition = maxEditValue; \
        if (lcdDrawUpdate) \
            lcd_implementation_drawedit(editLabel, _strFunc(((_type)encoderPosition) / scale)); \
        if (LCD_CLICKED) \
        { \
            *((_type*)editValue) = ((_type)encoderPosition) / scale; \
            lcd_quick_feedback(); \
            currentMenu = prevMenu; \
            encoderPosition = prevEncoderPosition; \
        } \
    } \
    void menu_edit_callback_ ## _name () \
    { \
        if ((int32_t)encoderPosition < minEditValue) \
            encoderPosition = minEditValue; \
        if ((int32_t)encoderPosition > maxEditValue) \
            encoderPosition = maxEditValue; \
        if (lcdDrawUpdate) \
            lcd_implementation_drawedit(editLabel, _strFunc(((_type)encoderPosition) / scale)); \
        if (LCD_CLICKED) \
        { \
            *((_type*)editValue) = ((_type)encoderPosition) / scale; \
            lcd_quick_feedback(); \
            currentMenu = prevMenu; \
            encoderPosition = prevEncoderPosition; \
            (*callbackFunc)();\
        } \
    } \
    static void menu_action_setting_edit_ ## _name (const char* pstr, _type* ptr, _type minValue, _type maxValue) \
    { \
        MENU_PAGE = 350; \
        prevMenu = currentMenu; \
        prevEncoderPosition = encoderPosition; \
         \
        lcdDrawUpdate = 2; \
        currentMenu = menu_edit_ ## _name; \
         \
        editLabel = pstr; \
        editValue = ptr; \
        minEditValue = minValue * scale; \
        maxEditValue = maxValue * scale; \
        encoderPosition = (*ptr) * scale; \
    }\
    static void menu_action_setting_edit_callback_ ## _name (const char* pstr, _type* ptr, _type minValue, _type maxValue, menuFunc_t callback) \
    { \
        prevMenu = currentMenu; \
        prevEncoderPosition = encoderPosition; \
         \
        lcdDrawUpdate = 2; \
        currentMenu = menu_edit_callback_ ## _name; \
         \
        editLabel = pstr; \
        editValue = ptr; \
        minEditValue = minValue * scale; \
        maxEditValue = maxValue * scale; \
        encoderPosition = (*ptr) * scale; \
        callbackFunc = callback;\
    }
*/
//menu_edit_type(int, int3, itostr3, 1)
//menu_edit_type(float, float3, ftostr3, 1)
//menu_edit_type(float, float32, ftostr32, 100)
//menu_edit_type(float, float5, ftostr5, 0.01)
//menu_edit_type(float, float51, ftostr51, 10)
//menu_edit_type(float, float52, ftostr52, 100)
//menu_edit_type(unsigned long, long5, ftostr5, 0.01)

/** End of menus **/

static void lcd_quick_feedback()
{
    lcdDrawUpdate = 2;
    blocking_enc = millis() + 500;
    lcd_implementation_quick_feedback();
}

/** Menu action functions **/
static void menu_action_back(menuFunc_t data)
{
    //SERIAL_PROTOCOLLN("menu_action_back");
    //SERIAL_ECHOPGM("page = ");
    //SERIAL_PROTOCOLLN((int)MENU_PAGE);
    
    currentMenu = data;
    encoderPosition = 0;
    
}
static void menu_action_submenu(menuFunc_t data)
{
    //SERIAL_PROTOCOLLN("menu_action_submenu");
    //SERIAL_ECHOPGM("page = ");
    //SERIAL_PROTOCOLLN((int)MENU_PAGE);

    currentMenu = data;
    encoderPosition = 0;
}
static void menu_action_gcode(const char* pgcode)
{
    enquecommand_P(pgcode);
}
static void menu_action_function(menuFunc_t data)
{
    (*data)();
}
static void menu_action_sdfile(const char* filename, char* longFilename)
{
    char cmd[30];
    char* c;
    sprintf_P(cmd, PSTR("M23 %s"), filename);
    for(c = &cmd[4]; *c; c++)
        *c = tolower(*c);
    enquecommand(cmd);
    enquecommand_P(PSTR("M24"));
    lcd_return_to_status();
}
static void menu_action_sddirectory(const char* filename, char* longFilename)
{
    card.chdir(filename);
    encoderPosition = 0;
}
static void menu_action_setting_edit_bool(const char* pstr, bool* ptr)
{
    *ptr = !(*ptr);
}
#endif//ULTIPANEL

/** LCD API **/
void lcd_init()
{
	
    lcd_implementation_init();

/*
    //2014/04/17 ABEE-SW
    pinMode(SW_LEFT,INPUT);
    pinMode(SW_RIGHT,INPUT);
    pinMode(SW_UP,INPUT);
    pinMode(SW_DOWN,INPUT);
    pinMode(SW_HOME,INPUT);
    pinMode(SW_ENTER,INPUT);
    WRITE(SW_LEFT,HIGH);
    WRITE(SW_RIGHT,HIGH);
    WRITE(SW_UP,HIGH);
    WRITE(SW_DOWN,HIGH);
    WRITE(SW_HOME,HIGH);
    WRITE(SW_ENTER,HIGH);
    //
*/

// 2014/07/08
#ifdef NEWPANEL
    pinMode(BTN_EN1,INPUT);
    pinMode(BTN_EN2,INPUT); 
    pinMode(BTN_ENC,INPUT);
    pinMode(BTN_LEFT,INPUT);
    pinMode(BTN_RIGHT,INPUT);    
    pinMode(SDCARDDETECT,INPUT);
    
    //2014/08/04
    //WRITE(BTN_EN1,HIGH);
    //WRITE(BTN_EN2,HIGH);
    //WRITE(BTN_ENC,HIGH);
    //WRITE(BTN_LEFT,HIGH);
    //WRITE(BTN_RIGHT,HIGH);
    //WRITE(BTN_HOME,HIGH);

    WRITE(BTN_EN1,LOW);
    WRITE(BTN_EN2,LOW);
    WRITE(BTN_ENC,LOW);
    WRITE(BTN_LEFT,LOW);
    WRITE(BTN_RIGHT,LOW);
    WRITE(BTN_HOME,LOW);
   ///

#else
    pinMode(SHIFT_CLK,OUTPUT);
    pinMode(SHIFT_LD,OUTPUT);
    pinMode(SHIFT_EN,OUTPUT);
    pinMode(SHIFT_OUT,INPUT);
    WRITE(SHIFT_OUT,HIGH);
    WRITE(SHIFT_LD,HIGH); 
    WRITE(SHIFT_EN,LOW);
#endif//!NEWPANEL
//

#if (SDCARDDETECT > -1)
    WRITE(SDCARDDETECT, HIGH);
    lcd_oldcardstatus = IS_SD_INSERTED;
#endif//(SDCARDDETECT > -1)


	//enquecommand_P(PSTR("G28"));

    lcd_buttons_update();
    encoderDiff = 0;
}


static bool boot_flg = false;

void lcd_update()
{
    if (boot_flg == false)
    {
        delay(BOOT_DELAY);
        boot_flg = true;
        //SERIAL_PROTOCOLLN("BOOT - DELAY");
    }
    
    static unsigned long timeoutToStatus = 0;
    
    
    if (SYS_COLD_ERR == true)
    {
        lcd.setCursor(0,0);
        lcd.print(SYS_ERR_COLD);   //Î(َĎ) ׎
        lcd.setCursor(0,1);
        lcd.print(SYS_PWR_OFF);   //ÎގݎގݎÎގ
        lcd.setCursor(-4,2);
        lcd.print("                ");
        lcd.setCursor(-4,3);
        lcd.print("                ");
        
        enquecommand_P(PSTR("M104 S0"));
        enquecommand_P(PSTR("M140 S0"));
        enquecommand_P(PSTR("M84"));
    }
    else
    {
        lcd_buttons_update();
        
        /*  2014/07/18
        #if (SDCARDDETECT > -1)
        if((IS_SD_INSERTED != lcd_oldcardstatus))
        {
            lcdDrawUpdate = 2;
            lcd_oldcardstatus = IS_SD_INSERTED;
            lcd_implementation_init(); // to maybe revive the lcd if static electricty killed it.
            
            if(lcd_oldcardstatus)
            {
                card.initsd();
                LCD_MESSAGEPGM(MSG_SD_INSERTED);
            }
            else
            {
                card.release();
                LCD_MESSAGEPGM(MSG_SD_REMOVED);
            }
        }
        #endif//CARDINSERTED
        */
        
        if (lcd_next_update_millis < millis())
        {
#ifdef ULTIPANEL
            if (encoderDiff)
            {
                ////
                //SERIAL_PROTOCOLPGM("encoderDiff ="); 
                //SERIAL_PROTOCOLLN(encoderDiff);
                //SERIAL_PROTOCOLLN("");
                ///

                lcdDrawUpdate = 1;
                //encoderPosition += encoderDiff;    //2014/04/18
                encoderDiff = 0;
                timeoutToStatus = millis() + LCD_TIMEOUT_TO_STATUS;
                
                //
                //SERIAL_PROTOCOLPGM("DIFF: encoderPosition ="); 
                //SERIAL_PROTOCOLLN((int)encoderPosition);
                //
            }
            else
            {
                //SERIAL_PROTOCOLPGM("ELSE: encoderPosition ="); 
                //SERIAL_PROTOCOLLN((int)encoderPosition);
                //SERIAL_PROTOCOLLN(prevEncoderPosition);
            }
            
/*        
            if (LCD_CLICKED)
                timeoutToStatus = millis() + LCD_TIMEOUT_TO_STATUS;
*/
#endif//ULTIPANEL

#ifdef DOGLCD        // Changes due to different driver architecture of the DOGM display
		    blink++;	   // Variable for fan animation and alive dot
		    u8g.firstPage();
		    do {
				u8g.setFont(u8g_font_6x10_marlin);
				u8g.setPrintPos(125,0);
				if (blink % 2) u8g.setColorIndex(1); else u8g.setColorIndex(0); // Set color for the alive dot
				u8g.drawPixel(127,63);	// draw alive dot
				u8g.setColorIndex(1);	// black on white
				(*currentMenu)();
				if (!lcdDrawUpdate)  break; // Terminate display update, when nothing new to draw. This must be done before the last dogm.next()
		    } while( u8g.nextPage() );
#else        
            (*currentMenu)();
#endif

#ifdef ULTIPANEL
            if(timeoutToStatus < millis() && currentMenu != lcd_main_menu /* 2014/07/10 lcd_status_screen */)
            {
                //SERIAL_ECHOPGM("SCAN CHECK-0");
                //2014/07/18
                //lcd_return_to_status();
                //lcdDrawUpdate = 2;
                if ((MENU_PAGE == 110) || (MENU_PAGE == 111 ) || (MENU_PAGE == 200) || (MENU_PAGE == 300) || 
                (MENU_PAGE == 301) || (MENU_PAGE == 302) || (MENU_PAGE == 401) || (MENU_PAGE == 402) || (MENU_PAGE == 999))
                {
                    //SERIAL_ECHOPGM("SCAN CHECK-1");
                    /*
                    if ((MENU_PAGE == 110) && (adjust_flg == true))
                    {
                        lcd_return_to_status();
                        lcdDrawUpdate = 2;
                        adjust_flg = false;
                    }
                    */
                }
                else
                {
                    //SERIAL_ECHOPGM("SCAN CHECK-2");
                    
                    if ((MENU_PAGE == 100) && (PUSH_SW == 4))
                    {
                        R_SW_CNT++;
                        lcdDrawUpdate = 1;
                        if (R_SW_CNT > 160)
                        {
                            //PUSH_SW = 0;
                            R_SW_CNT = 0;
                            lcd_quick_feedback();
                            currentMenu = lcd_main_menu;
                            lcd_return_to_status();
                            lcdDrawUpdate = 2;
                        }
                    }
                    else if ((MENU_PAGE == 101) && (PUSH_SW == 4))
                    {
                        //SERIAL_ECHOPGM("RIGHT_SW(101) && PUSH_SW == 4");
                        //SERIAL_PROTOCOLLN("");
                        
                        R_SW_CNT++;
                        lcdDrawUpdate = 1;
                        if (R_SW_CNT > 160)
                        {
                            //PUSH_SW = 0;
                            R_SW_CNT = 0;
                            lcd_quick_feedback();
                            currentMenu = lcd_main_menu;
                            lcd_return_to_status();
                            lcdDrawUpdate = 2;
                        }
                    }
                    else if ((MENU_PAGE == 1) && (PUSH_SW == 4))
                    {
                        //SERIAL_ECHOPGM("RIGHT_SW(1) && PUSH_SW == 4");
                        //SERIAL_PROTOCOLLN("");
                        
                        R_SW_CNT++;
                        lcdDrawUpdate = 1;
                        if (R_SW_CNT > 160)
                        {
                            //PUSH_SW = 0;
                            R_SW_CNT = 0;
                            lcd_quick_feedback();
                            currentMenu = lcd_main_menu;
                            lcd_return_to_status();
                            lcdDrawUpdate = 2;
                        }
                    }
                    else
                    {
                        lcd_return_to_status();
                        lcdDrawUpdate = 2;
                    }
                }
            }
#endif//ULTIPANEL

            if (lcdDrawUpdate == 2)
                lcd_implementation_clear();
            if (lcdDrawUpdate)
                lcdDrawUpdate--;
            lcd_next_update_millis = millis() + 100;

        }
    }
}


void lcd_setstatus(const char* message)
{
    if (lcd_status_message_level > 0)
        return;
        
    strncpy(lcd_status_message, message, LCD_WIDTH);
    lcdDrawUpdate = 2;
}
void lcd_setstatuspgm(const char* message)
{
    if (lcd_status_message_level > 0)
        return;
    strncpy_P(lcd_status_message, message, LCD_WIDTH);
    lcdDrawUpdate = 2;
}
void lcd_setalertstatuspgm(const char* message)
{
    lcd_setstatuspgm(message);
    lcd_status_message_level = 1;
#ifdef ULTIPANEL
    lcd_return_to_status();
#endif//ULTIPANEL
}
void lcd_reset_alert_level()
{
    lcd_status_message_level = 0;
}


//static int LCD_PAGE = 0;
static bool SW_RELEASE = true;
//static bool SW_RELEASE1 = true;
//static bool XY_MOVE_MODE = false;
extern bool MOVE_HOME_OK;

//static unsigned long rtime = 0, ptime = 0;
//static  uint8_t newbutton=0;


#ifdef ULTIPANEL
/* Warning: This function is called from interrupt context */
void lcd_buttons_update()
{
#ifdef NEWPANEL
    
    ///
    uint8_t newbutton=0;
    uint8_t enc=0;
    uint8_t ctmp = 0;
    
    if ((encoderPosition - (int)(encoderPosition / 5) * 5) != 0 )
        encoderPosition += 1;
/*    
    if ((blocking_enc<millis()) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1)
       && (READ(BTN_LEFT) == 1) && (READ(BTN_RIGHT) == 1) && (READ(BTN_HOME) == 1))
    {
        SW_RELEASE = true;
        //PUSH_SW = 0;
    }
*/   

#if SW_METHOD == 3
    if (SW_RELEASE == true)
    {
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/LEFT/DOWN/UP/ENTER");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/LEFT/DOWN/UP");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/LEFT/DOWN/ENTER");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/LEFT/DOWN");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/LEFT/UP/ENTER");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/LEFT/UP");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/LEFT/ENTER");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/LEFT");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/DOWN/UP/ENTER");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/DOWN/UP");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/DOWN/ENTER");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/DOWN");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/LEFT/UP/ENTER");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/UP");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT/ENTER");
        }
        if ((READ(BTN_RIGHT) == 0) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("RIGHT");
        }
        

        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("LEFT/DOWN/UP/ENTER");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("LEFT/DOWN/UP");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("LEFT/DOWN/ENTER");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("LEFT/DOWN");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("LEFT/UP/ENTER");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("LEFT/UP");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("LEFT/ENTER");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 0) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("LEFT");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("DOWN/UP/ENTER");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("DOWN/UP");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("DOWN/ENTER");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("DOWN");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("LEFT/UP/ENTER");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 0) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("UP");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 0) )
        {
            SW_RELEASE = false;
            SERIAL_PROTOCOLLN("ENTER");
        }
        if ((READ(BTN_RIGHT) == 1) && (READ(BTN_LEFT) == 1) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 1) )
        {
            SW_RELEASE = false;
            //SERIAL_PROTOCOLLN("RELASE SW");
        }
    }

    if (/*(blocking_enc<millis()) &&*/ (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 1)
       && (READ(BTN_LEFT) == 1) && (READ(BTN_RIGHT) == 1) && (READ(BTN_HOME) == 1))
    {
        if (SW_RELEASE == false)
        {
            SW_RELEASE = true;
            SERIAL_PROTOCOLLN("SW_RELEASE");
        }
    }

#endif



#if SW_METHOD == 2
    if (SW_RELEASE == true) 
    {
        if (READ(BTN_RIGHT) == 0)
        {
            if (RIGHT_CNT < SW_TIMING)
            {
                RIGHT_CNT++;
            }
            else
            {
                SW_RELEASE = false;
                RIGHT_CNT = 0;
#if SW_MON == 1
                SERIAL_PROTOCOLLN("RIGHT");
#endif
                PUSH_SW = 4;
            }
        }
        else if (READ(BTN_EN1) == 0)
        {
            if (DOWN_CNT < SW_TIMING)
            {
                DOWN_CNT++;
            }
            else
            {
                //newbutton |= EN_A;
                SW_RELEASE = false;
                DOWN_CNT = 0;
#if SW_MON == 1
                SERIAL_PROTOCOLLN("DOWN");
#endif
                PUSH_SW = 1;
            }
        }
        else if (READ(BTN_EN2) == 0)
        {
            //UP SW(METHOD-1)
            if (UP_CNT < SW_TIMING)
            {
                UP_CNT++;
            }
            else
            {
                //newbutton |= EN_B;
                SW_RELEASE = false;
                UP_CNT = 0;
#if SW_MON == 1
                SERIAL_PROTOCOLLN("UP");
#endif
                PUSH_SW = 2;
            }
        }
        else if (READ(BTN_ENC) == 0)
        //else if( (blocking_enc<millis()) && (READ(BTN_ENC)== 0)) 
        {
            //ENTER SW(MOTHOD-1)
            if (ENTER_CNT < SW_TIMING)
            {
                ENTER_CNT++;
            }
            else /*if (PUSH_SW != 5)*/
            {
                //newbutton |= EN_C;
                SW_RELEASE = false;
                ENTER_CNT = 0;
#if SW_MON == 1
                SERIAL_PROTOCOLLN("ENTER");
#endif
                PUSH_SW = 5;
            }
        }
        //buttons = newbutton;
        else if (READ(BTN_LEFT) == 0)
        {
            if (LEFT_CNT < SW_TIMING)
            {
                LEFT_CNT++;
            }
            else
            {
                SW_RELEASE = false;
                LEFT_CNT = 0;
#if SW_MON == 1
                SERIAL_PROTOCOLLN("LEFT");
#endif
                PUSH_SW = 3;
            }
        }
        else if (READ(BTN_HOME) == 0)
        {
            if (HOME_CNT < SW_TIMING)
            {
                HOME_CNT++;
            }
            else
            {
                SW_RELEASE = false;
                HOME_CNT = 0;
#if SW_MON == 1
                SERIAL_PROTOCOLLN("HOME");
#endif
                PUSH_SW = 6;
            }
        }
    }
    //buttons = newbutton;

    if (/*(blocking_enc<millis()) &&*/ (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_ENC) == 1)
       && (READ(BTN_LEFT) == 1) && (READ(BTN_RIGHT) == 1) && (READ(BTN_HOME) == 1))
    {
        if (RELEASE_CNT < SW_TIMING)
            RELEASE_CNT++;
        else
        {
            RELEASE_CNT = 0;
            SW_RELEASE = true;
            //SERIAL_PROTOCOLLN("SW_RELEASE");
        }
    }
    
    //lastEncoderBits = enc;
    
#endif //SW_METHOD == 2

#if SW_METHOD == 1

    if (SW_RELEASE == true) 
    {
        //RIGHT SW(METOHD-1)
        if (READ(BTN_RIGHT) == 0)
        {
            if (RIGHT_CNT < SW_TIMING)
            {
                RIGHT_CNT++;
            }
            else
            {
                RIGHT_CNT = 0;
                
                SW_RELEASE = false;
                PUSH_SW = 4;
                
#if SW_MON == 1
                SERIAL_PROTOCOLLN("RIGHT");
                SERIAL_ECHOPGM("page = ");
                SERIAL_PROTOCOLLN((int)MENU_PAGE);
#endif
                
                mente[mente_no++] = 1;
                
                if (MENU_PAGE == 1)
                {
                    if (encoderPosition == 0)
                    {
                        if (READ(SDCARDDETECT) != 0)
                            card.cardOK = false;
                        
                        R_SW_CNT = 0;
                        RC = 0;
                        encoderPosition = 5;
                        sd_print_flg = false;
                        sd_file_pos_bak = -1;
                        sd_file_pos = 0;
                        lcd_quick_feedback();
                        currentMenu = lcd_sdcard_menu;
                    }
                    if (encoderPosition == 5)
                    {
                        //SERIAL_PROTOCOLLN("R-SW");
                        encoderDiff = 5;
                        R_SW_CNT = 0;
                        encoderPosition = 5;
                        sd_print_flg = false;
                        sd_file_pos_bak = -1;
                        sd_file_pos = 0;
                        RC = 0;
                        lcd_quick_feedback();
                        currentMenu = lcd_sdcard_menu;
                    }
                    if (encoderPosition == 10)
                    {
                        encoderPosition = 5;
                        lcd_quick_feedback();
                        currentMenu = lcd_filament_heating;
                    }
                    if (encoderPosition == 15)
                    {
                        encoderPosition = 5;
                        lcd_quick_feedback();
                        currentMenu = lcd_move_z_menu;
                    }
                    if (encoderPosition == 20)
                    {
                        encoderPosition = 5;
                        lcd_quick_feedback();
                        currentMenu = lcd_preheat_menu;
                    }
                    //2014/08/25
                    if (encoderPosition == 25)
                    {
                        encoderPosition = 5;
                        lcd_quick_feedback();
                        currentMenu = lcd_temp_monitor;
                    }
                }
                
                if (MENU_PAGE == 101)
                {
                    //SERIAL_PROTOCOLLN("RIGHT_SW(1) = 101 ret");
                    return;
                }
                
                if (MENU_PAGE == 300)
                {
                    lcd_cls_flg = false;
                    encoderPosition = 5;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_y_menu;
                }
                if (MENU_PAGE == 301)
                {
                    lcd_cls_flg = false;
                    encoderPosition = 15;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                }
                if (MENU_PAGE == 302)
                {
                    lcd_cls_flg = false;
                    encoderPosition = 15;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                }
            }
        } //READ(BTN_RIGHT) == 0
        //DOWN SW(MEHOD-1)
        else if (READ(BTN_EN1) == 0)
        {
            if (DOWN_CNT < SW_TIMING)
            {
                DOWN_CNT++;
            }
            else
            {
                //if (PUSH_SW != 1) //down
                //{
                DOWN_CNT = 0;
                
                SW_RELEASE = false; 
                newbutton |= EN_A;
                PUSH_SW = 1;
                
                encoderDiff = 5;
                
                switch(MENU_PAGE)
                {
                    case   1: //lcd_main_menu()
                        encoderPosition += 5;
                        if ( encoderPosition > 25)  //2014/08/25 20->25
                            encoderPosition = 25;
                        break;
                        
                    case 100:  //print card ok
                        if (SD_FILE_COUNT > 3)
                        {
                            encoderPosition += 5;
                            if ( encoderPosition > 20)
                                encoderPosition = 20;
                            sd_file_pos++;
                            if ( sd_file_pos > SD_FILE_COUNT - 1)
                                sd_file_pos = SD_FILE_COUNT - 1;
                        }
                        else
                        {
                            if (sd_file_pos < SD_FILE_COUNT - 1)
                            {
                                encoderPosition += 5;
                                sd_file_pos++;
                            }
                            if (SD_FILE_COUNT == 0)
                                sd_file_pos = 0;
                        }
                        break;
                        
                    case 101: //print card fail
                        break;
                        
                    case 111:
                        if (encoderPosition == 10) 
                            encoderPosition = 15;
                        break;
                        
                    case 120: //print adjust
                        encoderPosition += 5;
                        if ( encoderPosition > 30)
                            encoderPosition = 30;
                        break;
                        
                    case 121:  //adjust nozzle temp
                        if (target_temperature[0] > 0)
                        {
                            encoderDiff = -1;
                            target_temperature[0]--;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0])));//degTargetHotend(0) + 0.5)));
                            enquecommand(cmd);
                        }
                        break;
                        
                    case 122:  //adjust bed temp
                        if ( target_temperature_bed > 0) 
                        {
                            encoderDiff = -1;
                            target_temperature_bed--;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed) /*int(degTargetBed() + 0.5)*/));
                            enquecommand(cmd);
                        }
                        break;
                        
                    case 123:  //adjust print speed
                        if (print_speed > 25)
                        {
                            encoderDiff = -1;
                            print_speed--;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M220 S%s"), itostr3left(print_speed));
                            enquecommand(cmd);
                        }
                        break;
                        
                    case 124:  //adjust gear speed
                        if (gear_speed > 50)
                        {
                            encoderDiff = -1;
                            gear_speed--;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M221 S%s"), itostr3left(gear_speed));
                            enquecommand(cmd);
                        }
                        break;
                        
                    case 125:  //adjust fan speed
                        if (adjust_fan_speed > 0)
                        {
                            encoderDiff = -1;
                            adjust_fan_speed--;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M106 S%s"), itostr3left(int((adjust_fan_speed * 255 ) / 100)));
                            enquecommand(cmd);
                        }
                        break;
                        
                    case 126:  //adjust print finish
                        encoderPosition += 5;
                        if ( encoderPosition > 10)
                            encoderPosition = 10;
                        break;
                        
                    case 200: //filament mount
                        SERIAL_ECHOPGM("case 200 ");
                        SERIAL_PROTOCOLLN(int(degHotend(0) + 0.5));
                        if (int(degHotend(0) + 0.5) > EXTRUDE_MINTEMP )
                        {
                            char cmd[30], cmd2[10];
                            sprintf_P(cmd, PSTR("G1 E%s F"), itostr3left(FILAMENT_MOVE));
                            sprintf_P(cmd2,PSTR("%s"), itostr3left(FILAMENT_FEEDRATE));
                            for( int i = 0; i < 30; i++)
                            {
                                //SERIAL_ECHOPGM("cmd[] = ");
                                //SERIAL_PROTOCOLLN((int)cmd[i]);
                                if (cmd[i] == 0)
                                {
                                    cmd[i] = cmd2[0];
                                    cmd[i+1] = cmd2[1];
                                    cmd[i+2] = cmd2[2];
                                    cmd[i+3] = '\0';
                                    break;
                                }
                            }
                            enquecommand_P(PSTR("G91"));
                            enquecommand(cmd);
                            //enquecommand_P(PSTR("G1 E50 F100"));
                            enquecommand_P(PSTR("G90"));
                            //enquecommand_P(PSTR("M114"));
                        }
                        else
                        {
                            //SERIAL_PROTOCOLLN("case 200 NG");
                        }
                        break;
                        
                    case 300: //xmove
                        if (t_xmove == 0)
                        {
                            //50mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 X-50 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_xmove == 1)
                        {
                            //10mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 X-10 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_xmove == 2)
                        {
                            //1mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 X-1 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        break;
                        
                    case 301: //ymove
                        if (t_ymove == 0)
                        {
                            //50mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Y-50 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_ymove == 1)
                        {
                            //10mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Y-10 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_ymove == 2)
                        {
                            //1mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Y-1 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        break;
                        
                    case 302: //zmove
                        if (t_zmove == 0)
                        {
                            //10mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Z50 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_zmove == 1)
                        {
                            //1mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Z10 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_zmove == 2)
                        {
                            //0.1mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Z1 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        break;
                        
                    case 303:
                        encoderPosition += 5;
                        if ( encoderPosition > 20)
                            encoderPosition = 20;
                        t_xmove = encoderPosition / 5 - 1;
                        break;
                        
                    case 304:
                        encoderPosition += 5;
                        if ( encoderPosition > 20)
                            encoderPosition = 20;
                        t_ymove = encoderPosition / 5 - 1;
                        break;
                        
                    case 305:
                        encoderPosition += 5;
                        if ( encoderPosition > 20)
                            encoderPosition = 20;
                        t_zmove = encoderPosition / 5 - 1;
                        break;
                        
                    case 400: //preheat
                        encoderPosition += 5;
                        if ( encoderPosition > 15)
                            encoderPosition = 15;
                        break;
                        
                    case 401:  //nozzle
                        if (target_temperature[0] > 0)
                        {
                            encoderDiff = -1;
                            target_temperature[0]--;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0])/*(degTargetHotend(0) + 0.5)*/));
                            enquecommand(cmd);
                            if (target_temperature[0] == 0)
                                HEAT_OFF = true;
                        }
                        break;
                        
                    case 402: //bed_temp down
                        if ( target_temperature_bed > 0) 
                        {
                            encoderDiff = -1;
                            target_temperature_bed--;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed)/*int(degTargetBed() + 0.5)*/));
                            enquecommand(cmd);
                            if (target_temperature_bed == 0)
                                BED_OFF = true;
                        }
                        break;
                        
                    default:
                        break;
                        
                } //switch
                
#if SW_MON == 1
                SERIAL_PROTOCOLLN("DOWN");
                SERIAL_ECHOPGM("page = ");
                SERIAL_PROTOCOLLN((int)MENU_PAGE);
                //SERIAL_ECHOPGM("sd_file_pos= ");
                //SERIAL_PROTOCOLLN((int)sd_file_pos);
                //SERIAL_ECHOPGM("sd file count= ");
                //SERIAL_PROTOCOLLN(SD_FILE_COUNT);
                SERIAL_ECHOPGM("encoderPosition = ");
                SERIAL_PROTOCOLLN((int)encoderPosition);
                //SERIAL_PROTOCOLLN("encoderDiff= ");
                //SERIAL_PROTOCOLLN((int)encoderDiff);
#endif
                /*
                int u;
                if (sd_file_pos > 2)
                   u = sd_file_pos - 2;
                else
                   u = 0;
                for (int i = 0 ; i < 3; i++)
                {
                    card.getfilename(u + i);
                    //SERIAL_ECHOPGM("[");
                    //SERIAL_PROTOCOL(i+u)
                    //SERIAL_ECHOPGM("] = ");
                    //SERIAL_PROTOCOLLN(card.filename);
                }
                */
            } //DOWN SW 
        } //READ(BTN_EN1) == 0
        else if (READ(BTN_EN2) == 0)
        {
            //UP SW(METHOD-1)
            if (UP_CNT < SW_TIMING)
            {
                UP_CNT++;
            }
            else
            {
                //if (PUSH_SW != 2) //up
                //{
                UP_CNT = 0;
                
                SW_RELEASE = false; 
                newbutton |= EN_B;
                PUSH_SW = 2;
                
                mente[mente_no++] = 3;
                
                if ((encoderPosition - (int)(encoderPosition / 5) * 5) != 0 )
                    encoderPosition += 1;
                
                switch(MENU_PAGE)
                {
                    case 100:  //print card ok
                        encoderDiff = -5;
                        if (encoderPosition > 0)
                            encoderPosition -= 5;
                        
                        sd_file_pos--;
                        if ( sd_file_pos < 0)
                            sd_file_pos = 0;
                        break;
                        
                    case 101: //print card fail
                        break;
                        
                    case 111:
                         if (encoderPosition == 15) 
                             encoderPosition = 10;
                         break;
                         
                    case 121:  //adjust nozzle temp
                        if (target_temperature[0] < HEAT_TEMP_MAX)
                        {
                            encoderDiff = 1;
                            target_temperature[0]++;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0]/*degTargetHotend(0) + 0.5*/)));
                            enquecommand(cmd);
                        }
                        break;
                        
                    case 122:  //adjust bed temp
                        if ( target_temperature_bed < BED_TEMP_MAX) 
                        {
                            encoderDiff = 1;
                            target_temperature_bed++;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed/*degTargetBed() + 0.5*/)));
                            enquecommand(cmd);
                        }
                        break;
                        
                    case 123:  //adjust print speed
                        if (print_speed < PRINT_SPEED_MAX)
                        {
                            encoderDiff = 1;
                            print_speed++;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M220 S%s"), itostr3left(print_speed));
                            enquecommand(cmd);
                        }
                        break;
                        
                    case 124:  //adjust gear speed
                        if (gear_speed < 150)
                        {
                            encoderDiff = 1;
                            gear_speed++;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M221 S%s"), itostr3left(gear_speed));
                            enquecommand(cmd);
                        }
                        break;
                        
                    case 125:  //adjust fan speed
                        if (adjust_fan_speed < 100)
                        {
                            encoderDiff = 1;
                            adjust_fan_speed++;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M106 S%s"), itostr3left(int((adjust_fan_speed * 255 ) / 100)));
                            enquecommand(cmd);
                        }
                        break;
                        
                    case 126:  //adjust print finish
                        encoderDiff = -5;
                        if (encoderPosition > 0)
                            encoderPosition -= 5;
                        break;
                        
                    case 200: //filament remove
                        if (int(degHotend(0) + 0.5) > EXTRUDE_MINTEMP )
                        {
                            char cmd[30], cmd2[10];
                            sprintf_P(cmd, PSTR("G1 E-%s F"), itostr3left(FILAMENT_MOVE));
                            sprintf_P(cmd2,PSTR("%s"), itostr3left(FILAMENT_FEEDRATE));
                            for( int i = 0; i < 30; i++)
                            {
                                //SERIAL_ECHOPGM("cmd[] = ");
                                //SERIAL_PROTOCOLLN((int)cmd[i]);
                                if (cmd[i] == 0)
                                {
                                    cmd[i] = cmd2[0];
                                    cmd[i+1] = cmd2[1];
                                    cmd[i+2] = cmd2[2];
                                    cmd[i+3] = '\0';
                                    break;
                                }
                            }
                            enquecommand_P(PSTR("G91"));
                            enquecommand(cmd);
                            //enquecommand_P(PSTR("G1 E50 F100"));
                            enquecommand_P(PSTR("G90"));
                            //enquecommand_P(PSTR("M114"));
                        }
                        break;
                        
                    case 300: //xmove
                        if (t_xmove == 0)
                        {
                            //50mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 X50 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_xmove == 1)
                        {
                            //10mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 X10 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_xmove == 2)
                        {
                            //1mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 X1 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        break;
                        
                    case 301: //ymove
                        if (t_ymove == 0)
                        {
                            //50mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Y50 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_ymove == 1)
                        {
                            //10mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Y10 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_ymove == 2)
                        {
                            //1mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Y1 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        break;
                        
                    case 302: //zmove
                        if (t_zmove == 0)
                        {
                            //50mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Z-50 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_zmove == 1)
                        {
                            //10mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Z-10 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        if (t_zmove == 2)
                        {
                            //0.1mm
                            enquecommand_P(PSTR("G91"));
                            enquecommand_P(PSTR("G950 Z-1 F4800"));
                            enquecommand_P(PSTR("G90"));
                        }
                        break;
                        
                    case 303:
                        encoderDiff = -5;
                        if (encoderPosition > 5)
                            encoderPosition -= 5;
                        t_xmove = encoderPosition / 5 - 1;
                        break;
                        
                    case 304:
                        encoderDiff = -5;
                        if (encoderPosition > 5)
                            encoderPosition -= 5;
                        t_ymove = encoderPosition / 5 - 1;
                        break;
                        
                    case 305:
                        encoderDiff = -5;
                        if (encoderPosition > 5)
                            encoderPosition -= 5;
                        t_zmove = encoderPosition / 5 - 1;
                        break;
                        
                    case 350:  //unused
                        encoderDiff = -1;
                        //if (encoderPosition > 0)
                        encoderPosition--;
                        //if (encoderPosition < 0) encoderPosition = 0;
                        break;
                        
                    case 401: //nozzle_temp up
                        if ((HEAT_TEMP_MAX ) > target_temperature[0]) 
                        {
                            encoderDiff = 1;
                            target_temperature[0]++;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0]/*degTargetHotend(0) + 0.5*/)));
                            enquecommand(cmd);
                            
                            HEAT_OFF = false;
                        }
                        break;
                        
                    case 402: //bed_temp up
                        if (BED_TEMP_MAX > target_temperature_bed) 
                        {
                            encoderDiff = 1;
                            target_temperature_bed++;
                            char cmd[30];
                            sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed/*degTargetBed() + 0.5*/)));
                            enquecommand(cmd);
                            
                            BED_OFF = false;
                        }
                        break;
                        
                    default:
                        encoderDiff = -5;
                        if (encoderPosition > 0)
                            encoderPosition -= 5;
                        break;
                        
                }
                
#if SW_MON == 1
                SERIAL_PROTOCOLLN("UP");
                SERIAL_ECHOPGM("page = ");
                SERIAL_PROTOCOLLN((int)MENU_PAGE);
                SERIAL_ECHOPGM("encoderPosition = ");
                SERIAL_PROTOCOLLN((int)encoderPosition);
                //SERIAL_ECHOPGM("sd_file_pos= ");
                //SERIAL_PROTOCOLLN((int)sd_file_pos);
                //SERIAL_ECHOPGM("sd file count= ");
                //SERIAL_PROTOCOLLN(SD_FILE_COUNT);
                //SERIAL_PROTOCOLLN("encoderDiff= ");
                //SERIAL_PROTOCOLLN((int)encoderDiff);
#endif

            } //UP SW 
        
        } //READ(BTN_EN2) == 0
    } //SW_RELEASE
    
    
    //ENTER SW
    if( (blocking_enc<millis()) && (READ(BTN_ENC)== 0)) 
    //if(READ(BTN_ENC) == 0)
    {
        //delay(2);
        //if (READ(BTN_ENC) != 0)
        //   return;
        
        if (PUSH_SW != 5) 
        {
            SW_RELEASE = false;
            encoderDiff = -1;
            bak_encpos = encoderPosition;
            
            PUSH_SW = 5;
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_ENTER(0)");
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            SERIAL_PROTOCOLPGM("encoderPosition =");
            SERIAL_PROTOCOLLN((int)encoderPosition);
            //SERIAL_PROTOCOLPGM("bak_encpos =");
            //SERIAL_PROTOCOLLN((int)bak_encpos);
            //SERIAL_PROTOCOLPGM("encoderDiff= ");
            //SERIAL_PROTOCOLLN((int)encoderDiff);
#endif
            
            /*
            //SD print
            if ((MENU_PAGE == 100) && (sd_print_flg  == false))
            {
                sd_print_flg  = true;
                lcd_cls_flg = false;
                
                SERIAL_PROTOCOLLN("PRINT GOGO(1) ");
                card.getfilename(sd_file_pos);
                
                //card.getWorkDirName();
                if (card.filenameIsDir)
                {
                    //folder
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    
                    //SERIAL_PROTOCOLPGM("ENTER::FOLDER = ");
                    //SERIAL_PROTOCOLLN(card.filename);
                    card.chdir(card.filename);
                    card.getWorkDirName();
                    //SERIAL_PROTOCOLPGM("Workdir = ");
                    //SERIAL_PROTOCOLLN(card.getWorkDirName());
                    
                    lcd_quick_feedback();
                    currentMenu = lcd_sdcard_menu;
                }
                else
                {
                    if (Fcnt == 0)
                    {
                        lcd_quick_feedback();
                        currentMenu = lcd_sdcard_menu;
                    }
                    else
                    {
                        char cmd[30];
                        char* c;
                        sprintf_P(cmd, PSTR("M23 %s"), card.filename);
                        for(c = &cmd[4]; *c; c++)
                            *c = tolower(*c);
                        enquecommand(cmd);
                        //enquecommand_P(PSTR("M17"));  //stepper all on
                        
                        enquecommand_P(PSTR("M24"));
                        
                        //delay(20);  //open٤
                        //SERIAL_PROTOCOLPGM("PRINT IS_SD_PRINTING =  ");
                        //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                        //enquecommand_P(PSTR("M27"));
                        
                        lcd_quick_feedback();
                        currentMenu = lcd_print_screen;
                    }
                }
            }
            */
            
            
            if (MENU_PAGE == 1)
            {
                if (encoderPosition == 5)
                {
                    no_card_flg == false;
                    lcd_cls_flg = false;
                    RC = 0;
                    lcd_quick_feedback();
                    currentMenu = lcd_sdcard_menu;
                }
            }
            
            if (MENU_PAGE == 101)
            {
                //SERIAL_PROTOCOLLN("ENTER SW = 101 ret");
                return;
            }
            
            /*
            //2014/08/25
            if (MENU_PAGE == 101)
            {
                if (encoderPosition == 5)
                {
                    no_card_flg == false;
                    lcd_cls_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    //SERIAL_PROTOCOLLN("PAGE(0) = 101 ");
                    //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                }
            }
            ///
            */
            
            /*
            
            if (MENU_PAGE == 110)  //adjust
            {
                SERIAL_PROTOCOLPGM("Adjust(0) ");
            }
            
            if (MENU_PAGE == 300)
            {
                lcd_quick_feedback();
                currentMenu = lcd_xmove50;
            }
            if (MENU_PAGE == 301)
            {
                lcd_quick_feedback();
                currentMenu = lcd_ymove50;
            }
            if (MENU_PAGE == 302)
            {
                lcd_quick_feedback();
                currentMenu = lcd_zmove50;
            }
*/

/*
            if (MENU_PAGE == 111)
            {
                if (encoderPosition == 5)
                {
                    //No
                    //SERIAL_PROTOCOLLN("PRINT END(0) - TOP  ");
                    lcd_cls_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                }
                else
                {
                    //Yes
                    sd_print_flg  = true;
                    lcd_cls_flg = false;
                    
                    SERIAL_PROTOCOLLN("REPRINT  ");
                    card.getfilename(sd_file_pos);
                    
                    char cmd[30];
                    char* c;
                    sprintf_P(cmd, PSTR("M23 %s"), card.filename);
                    for(c = &cmd[4]; *c; c++)
                        *c = tolower(*c);
                    enquecommand(cmd);
                    //delay(20);
                    enquecommand_P(PSTR("M24"));
                    
                    lcd_quick_feedback();
                    currentMenu = lcd_print_screen;
                }
            }
*/
            //adjust
            if (MENU_PAGE == 110)
            {
                //PUSH_SW = 0;
                //adjust_flg = true;
                //lcd_quick_feedback();
                currentMenu = lcd_print_adjust_menu;
            }
            
            
            //x move
            if (MENU_PAGE == 303)
            {
                if (t_xmove != 3)
                {
                    xcmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                }
                else
                {
                    if (xcmd_flg == false)
                    {
                        xcmd_flg = true;
                        enquecommand_P(PSTR("G28 X0"));
                    }
                }
            }
            //y move
            if (MENU_PAGE == 304)
            {
                if (t_ymove != 3)
                {
                    ycmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_y_menu;
                }
                else
                {
                    if (ycmd_flg == false)
                    {
                        ycmd_flg = true;
                        enquecommand_P(PSTR("G28 X0"));
                    }
                }
            }
            //z move
            if (MENU_PAGE == 305)
            {
                if (t_zmove != 3)
                {
                    zcmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                }
                else
                {
                    if (zcmd_flg == false)
                    {
                        zcmd_flg = true;
                        enquecommand_P(PSTR("G28 Z0"));
                    }
                }
            }
        }
        
        else if (SW_RELEASE == true)
        {
            SERIAL_PROTOCOLLN("SW_ENTER(1)");
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            SERIAL_PROTOCOLPGM("encoderPosition =");
            SERIAL_PROTOCOLLN((int)encoderPosition);
            
            SW_RELEASE = false;
            
            //encoderDiff = -1;
            bak_encpos = encoderPosition;
            
            //SD print
            if ((MENU_PAGE == 100) && (sd_print_flg  == false))
            {
                sd_print_flg  = true;
                lcd_cls_flg = false;
                FINISH_PROC = false;
                
                
                //SERIAL_PROTOCOLLN("PRINT GOGO(1) ");
                card.getfilename(sd_file_pos);
                
                //card.getWorkDirName();
                if (card.filenameIsDir)
                {
                    //folder
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    
                    //SERIAL_PROTOCOLPGM("ENTER::FOLDER = ");
                    //SERIAL_PROTOCOLLN(card.filename);
                    card.chdir(card.filename);
                    card.getWorkDirName();
                    //SERIAL_PROTOCOLPGM("Workdir = ");
                    //SERIAL_PROTOCOLLN(card.getWorkDirName());
                    RC = 0;
                    lcd_quick_feedback();
                    currentMenu = lcd_sdcard_menu;
                }
                else
                {
                    if (Fcnt == 0)
                    {
                        RC = 0;
                        lcd_quick_feedback();
                        currentMenu = lcd_sdcard_menu;
                    }
                    else
                    {
                        char cmd[30];
                        char* c;
                        sprintf_P(cmd, PSTR("M23 %s"), card.filename);
                        for(c = &cmd[4]; *c; c++)
                            *c = tolower(*c);
                        enquecommand(cmd);
                        //enquecommand_P(PSTR("M17"));  //stepper all on
                        
                        enquecommand_P(PSTR("M24"));
                        
                        //delay(20);  //open٤
                        //SERIAL_PROTOCOLPGM("PRINT IS_SD_PRINTING =  ");
                        //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                        //enquecommand_P(PSTR("M27"));
                        
                        lcd_quick_feedback();
                        currentMenu = lcd_print_screen;
                    }
                }
            }
            
            if (MENU_PAGE == 101)
            {
                //SERIAL_PROTOCOLLN("ENTER SW = 101 ret");
                return;
            }
            
            //2014/08/25
            /*
            if (MENU_PAGE == 101)
            {
                if (encoderPosition == 5)
                {
                    no_card_flg == false;
                    lcd_cls_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    //SERIAL_PROTOCOLLN("PAGE(1) = 101 ");
                }
            }
            ///
            */
            
            
            //adjust
            if (MENU_PAGE == 110)
            {
                PUSH_SW = 0;
                encoderPosition = 5;
                //adjust_flg = true;
                lcd_quick_feedback();
                currentMenu = lcd_print_adjust_menu;
            }
            
            //print finish
            if (MENU_PAGE == 111)
            {
                if (encoderPosition == 10)
                {
                    //No
                    //SERIAL_PROTOCOLLN("PRINT END(1) - TOP  ");
                    encoderPosition = 5;
                    lcd_cls_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_finish_proc; //lcd_main_menu;
                }
                else if ((encoderPosition == 15) && (sd_print_flg == false))
                {
                    //Yes
                    sd_print_flg  = true;
                    lcd_cls_flg = false;
                    
                    //SERIAL_PROTOCOLLN("RE-PRINT  ");
                    card.getfilename(sd_file_pos);
                    
                    char cmd[30];
                    char* c;
                    sprintf_P(cmd, PSTR("M23 %s"), card.filename);
                    for(c = &cmd[4]; *c; c++)
                        *c = tolower(*c);
                    enquecommand(cmd);
                    enquecommand_P(PSTR("M17"));

                    //delay(20);  //open٤
                    enquecommand_P(PSTR("M24"));
                    
                    //SERIAL_PROTOCOLPGM("PRINT IS_SD_PRINTING =  ");
                    //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                    
                    lcd_quick_feedback();
                    currentMenu = lcd_print_screen;
                }
            }
            
            //adjust nozzle
            if (MENU_PAGE == 120)
            {
                if (encoderPosition == 5)
                {
                    //nozzle
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_nozzle;
                }
                if (encoderPosition == 10)
                {
                    //bed temp
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_bed_temp;
                }
                if (encoderPosition == 15)
                {
                    //print speed
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_print_speed;
                }
                if (encoderPosition == 20)
                {
                    //gear speed
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_gear_speed;
                }
                if (encoderPosition == 25)
                {
                    //fan speed
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_fan_speed;
                }
                if (encoderPosition == 30)
                {
                    //print fin.
                    encoderPosition = 5;
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_print_finish;
                }
            }
            
            //kill print
            if (MENU_PAGE == 126) //print fin.
            {
                //if (IS_SD_PRINTING == false)
                //    return;
                
                if (encoderPosition == 5)
                {
                    lcd_quick_feedback();
                    currentMenu = lcd_print_screen;
                }
                else
                {
                    //if (IS_SD_PRINTING == true)
                    {
                        //λ
                        //lcd_sdcard_stop();
                        
                        //SERIAL_PROTOCOLPGM("PRINT IS_SD_PRINTING =  ");
                        //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                        
                        sd_print_flg = false;
                        exec_print = false;
                        card.sdprinting = false;
                        
                        PUSH_SW = 0;
                        encoderPosition = 5;
                        //lcdDrawUpdate = 2;
                        lcd_cls_flg = false;
                        lcd_quick_feedback();
                        currentMenu = lcd_print_finish_proc;
                        
                        //card.cardOK = false;
                        //st_synchronize();
                        //quickStop();
                        //card.closefile();
                        
                        //enquecommand_P(PSTR("M22"));   //release SD
                        //enquecommand_P(PSTR("M84"));   //all stepper disable
                        
                        //if(SD_FINISHED_STEPPERRELEASE)
                        //{
                        //    enquecommand_P(PSTR(SD_FINISHED_RELEASECOMMAND));
                        //}
                        //autotempShutdown();
                        
                        /*
                        for (;;)
                        {
                            //if (card.cardOK == false)
                            {
                                //SERIAL_PROTOCOLPGM("PRINT IS_SD_PRINTING =  ");
                                //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                                SERIAL_PROTOCOLPGM("SD status = ");
                                enquecommand_P(PSTR("M27"));
                                break;
                            }
                        }
                        */
                        
                        
                    }
                }
            }
            
            if (MENU_PAGE == 300)
            {
                lcd_quick_feedback();
                currentMenu = lcd_xmove50;
            }
            if (MENU_PAGE == 301)
            {
                lcd_quick_feedback();
                currentMenu = lcd_ymove50;
            }
            if (MENU_PAGE == 302)
            {
                lcd_quick_feedback();
                currentMenu = lcd_zmove50;
            }
            if (MENU_PAGE == 303)
            {
                if (t_xmove != 3)
                {
                    xcmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                }
                else
                {
                    if (xcmd_flg == false)
                    {
                        xcmd_flg = true;
                        enquecommand_P(PSTR("G28 X0"));
                    }
                }
            }
            if (MENU_PAGE == 304)
            {
                if (t_ymove != 3)
                {
                    ycmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_y_menu;
                }
                else
                {
                    if (ycmd_flg == false)
                    {
                        ycmd_flg = true;
                        enquecommand_P(PSTR("G28 X0"));
                    }
                }
            }
            if (MENU_PAGE == 305)
            {
                if (t_zmove != 3)
                {
                    zcmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                }
                else
                {
                    if (zcmd_flg == false)
                    {
                        zcmd_flg = true;
                        enquecommand_P(PSTR("G28 Z0"));
                    }
                }
            }
        }
        
        newbutton |= EN_C;
    }
    buttons = newbutton;
    
    //LEFT SW
    if ( (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_LEFT) == 0) && (READ(BTN_RIGHT) == 1) )
    //if(READ(BTN_LEFT) == 0)
    {
        delay(2);
        if (READ(BTN_LEFT) != 0)
            return;

        if (PUSH_SW != 3)
        {
            SW_RELEASE = false;
            PUSH_SW = 3; 
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_LEFT(0)");
#endif
            
            mente[mente_no++] = 2;
            
            if (MENU_PAGE == 1)
                return;
            
            switch (MENU_PAGE)
            {
                case 100:  //print card ok
                    sd_print_flg = false;
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    card.setroot();
                    
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 101:  //print card fail
                    sd_print_flg = false;
                    no_card_flg = false;    //2014/08/25
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    card.setroot();
                    
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 120:
                    lcd_cls_flg = false;
                    encoderPosition = 5;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_screen;
                    break;
                    
                case 121:  //print adjust nozzle
                    lcd_cls_flg = false;
                    encoderPosition = 5;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 122:  //print adjust bed
                    lcd_cls_flg = false;
                    encoderPosition = 10;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 123:  //print adjust print speed
                    lcd_cls_flg = false;
                    encoderPosition = 15;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 124:  //print adjust gear speed
                    lcd_cls_flg = false;
                    encoderPosition = 20;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 125:  //print adjust fan speed
                    lcd_cls_flg = false;
                    encoderPosition = 25;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 126:  //print adjust print speed
                    lcd_cls_flg = false;
                    encoderPosition = 30;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 200:  //Filament
                    pre_heat_flg = false;
                    enquecommand_P(PSTR("M104 S0"));  //extruder heat off
                    encoderPosition = 10;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 300:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                    break;
                    
                case 301:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                    break;
                    
                case 302:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 303:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                    break;
                    
                case 304:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_y_menu;
                    break;
                    
                case 305:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                    break;
                    
                case 400:  //preheat
                    lcd_cls_flg = false;
                    encoderPosition = 20;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 401:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                case 402:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 10;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                case 500:  //temp monitor
                    lcd_cls_flg = false;
                    encoderPosition = 25;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                default:
                    break;
            }
            //lcdDrawUpdate = 2;
            //lcd_next_update_millis = millis() + 100;
            
#if SW_MON == 1
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            //SERIAL_PROTOCOLPGM("encoderPosition =");
            //SERIAL_PROTOCOLLN((int)encoderPosition);
#endif
        }
        else if (SW_RELEASE == true)
        {
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_LEFT(1)");
#endif
            
            SW_RELEASE = false;
            
            if (MENU_PAGE == 1)
                return;
            
            switch (MENU_PAGE)
            {
                case 100:  //print card ok
                    sd_print_flg = false;
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    card.setroot();
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 101:  //print card fail
                    sd_print_flg = false;
                    no_card_flg = false;    //2014/08/25
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    card.setroot();
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 120:
                    lcd_cls_flg = false;
                    encoderPosition = 5;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_screen;
                    break;
                    
                case 121:  //print adjust nozzle
                    lcd_cls_flg = false;
                    encoderPosition = 5;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 122:  //print adjust bed
                    lcd_cls_flg = false;
                    encoderPosition = 10;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 123:  //print adjust print speed
                    lcd_cls_flg = false;
                    encoderPosition = 15;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 124:  //print adjust gear speed
                    lcd_cls_flg = false;
                    encoderPosition = 20;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 125:  //print adjust fan speed
                    lcd_cls_flg = false;
                    encoderPosition = 25;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 126:  //print adjust print speed
                    lcd_cls_flg = false;
                    encoderPosition = 30;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 200:  //Filament
                    pre_heat_flg = false;
                    enquecommand_P(PSTR("M104 S0"));  //extruder heat off
                    encoderPosition = 10;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 300:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                    break;
                    
                case 301:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                    break;
                    
                case 302:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 303:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                    break;
                    
                case 304:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_y_menu;
                    break;
                    
                case 305:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                    break;
                    
                case 400:  //preheat
                    lcd_cls_flg = false;
                    encoderPosition = 20;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 401:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                case 402:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 10;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                case 500:  //temp monitor
                    lcd_cls_flg = false;
                    encoderPosition = 25;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                default:
                    break;
            }
            //lcdDrawUpdate = 2;
            //lcd_next_update_millis = millis() + 100;
            
            
#if SW_MON == 1
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            //SERIAL_PROTOCOLPGM("encoderPosition =");
            //SERIAL_PROTOCOLLN((int)encoderPosition);
#endif
        }
    }
    
    
    //home switch
    if (READ(BTN_HOME) == 0)
    {
        delay(2);
        if (READ(BTN_HOME) != 0)
            return;

        if (PUSH_SW != 6) //down
        {
            SW_RELEASE = false; 
            PUSH_SW = 6;
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_HOME(0)");
#endif
            
            MOVE_HOME_OK = false;
            enquecommand_P(PSTR("G28"));
            return;
        }
        else if (SW_RELEASE == true)
        {
            SW_RELEASE = false;
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_HOME(1)");
#endif
        }
    }
    
    if ((blocking_enc<millis()) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1)
       && (READ(BTN_LEFT) == 1) && (READ(BTN_RIGHT) == 1) && (READ(BTN_HOME) == 1))
    {
        SW_RELEASE = true;
        //PUSH_SW = 0;
    }




#endif //SW_METHOD == 1 
    
#if SW_METHOD == 0
    //DOWN SW 
    if ( (READ(BTN_EN1) == 0) && (READ(BTN_EN2) == 1) && (READ(BTN_LEFT) == 1) && (READ(BTN_RIGHT) == 1) )    
    {
        delay(2);
        if (READ(BTN_EN1) != 0)
            return;
        
        if (PUSH_SW != 1) //down
        {
            DOWN_CNT = 0;

            SW_RELEASE = false; 
            newbutton |= EN_A;
            PUSH_SW = 1;
            
            encoderDiff = 5;
            
            switch(MENU_PAGE)
            {
                case   1: //lcd_main_menu()
                    encoderPosition += 5;
                    if ( encoderPosition > 25)  //2014/08/25 20->25
                        encoderPosition = 25;
                    break;
                    
                case 100:  //print card ok
                    if (SD_FILE_COUNT > 3)
                    {
                        encoderPosition += 5;
                        if ( encoderPosition > 20)
                            encoderPosition = 20;
                        sd_file_pos++;
                        if ( sd_file_pos > SD_FILE_COUNT - 1)
                            sd_file_pos = SD_FILE_COUNT - 1;
                    }
                    else
                    {
                        if (sd_file_pos < SD_FILE_COUNT - 1)
                        {
                            encoderPosition += 5;
                            sd_file_pos++;
                        }
                        if (SD_FILE_COUNT == 0)
                            sd_file_pos = 0;
                    }
                    
                        
                    //if (sd_file_pos > SD_FILE_COUNT)
                    //{
                    //    sd_file_pos = SD_FILE_COUNT - 1;
                    //    encoderPosition = SD_FILE_COUNT * 5;
                    //}
                    break;
                    
                case 101: //print card fail
                    break;
                    
                case 111:
                    if (encoderPosition == 10) 
                        encoderPosition = 15;
                    break;
                     
                case 120: //print adjust
                    encoderPosition += 5;
                    if ( encoderPosition > 30)
                        encoderPosition = 30;
                    break;
                    
                case 121:  //adjust nozzle temp
                    if (target_temperature[0] > 0)
                    {
                        encoderDiff = -1;
                        target_temperature[0]--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0])));//degTargetHotend(0) + 0.5)));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 122:  //adjust bed temp
                    if ( target_temperature_bed > 0) 
                    {
                        encoderDiff = -1;
                        target_temperature_bed--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed) /*int(degTargetBed() + 0.5)*/));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 123:  //adjust print speed
                    if (print_speed > 25)
                    {
                        encoderDiff = -1;
                        print_speed--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M220 S%s"), itostr3left(print_speed));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 124:  //adjust gear speed
                    if (gear_speed > 50)
                    {
                        encoderDiff = -1;
                        gear_speed--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M221 S%s"), itostr3left(gear_speed));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 125:  //adjust fan speed
                    if (adjust_fan_speed > 0)
                    {
                        encoderDiff = -1;
                        adjust_fan_speed--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M106 S%s"), itostr3left(int((adjust_fan_speed * 255 ) / 100)));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 126:  //adjust print finish
                    encoderPosition += 5;
                    if ( encoderPosition > 10)
                        encoderPosition = 10;
                    break;
                    
                case 200: //filament mount
                    SERIAL_ECHOPGM("case 200 ");
                    SERIAL_PROTOCOLLN(int(degHotend(0) + 0.5));
                    if (int(degHotend(0) + 0.5) > EXTRUDE_MINTEMP )
                    {
                        char cmd[30], cmd2[10];
                        sprintf_P(cmd, PSTR("G1 E%s F"), itostr3left(FILAMENT_MOVE));
                        sprintf_P(cmd2,PSTR("%s"), itostr3left(FILAMENT_FEEDRATE));
                        for( int i = 0; i < 30; i++)
                        {
                            //SERIAL_ECHOPGM("cmd[] = ");
                            //SERIAL_PROTOCOLLN((int)cmd[i]);
                            if (cmd[i] == 0)
                            {
                                cmd[i] = cmd2[0];
                                cmd[i+1] = cmd2[1];
                                cmd[i+2] = cmd2[2];
                                cmd[i+3] = '\0';
                                break;
                            }
                        }
                        enquecommand_P(PSTR("G91"));
                        enquecommand(cmd);
                        //enquecommand_P(PSTR("G1 E50 F100"));
                        enquecommand_P(PSTR("G90"));
                        //enquecommand_P(PSTR("M114"));
                    }
                    else
                    {
                        //SERIAL_PROTOCOLLN("case 200 NG");
                    }
                    break;
                    
                case 300: //xmove
                    if (t_xmove == 0)
                    {
                        //50mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X-50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_xmove == 1)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X-10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_xmove == 2)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X-1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 301: //ymove
                    if (t_ymove == 0)
                    {
                        //50mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y-50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_ymove == 1)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y-10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_ymove == 2)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y-1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 302: //zmove
                    if (t_zmove == 0)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_zmove == 1)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_zmove == 2)
                    {
                        //0.1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 303:
                    encoderPosition += 5;
                    if ( encoderPosition > 20)
                        encoderPosition = 20;
                    t_xmove = encoderPosition / 5 - 1;
                    break;
                    
                case 304:
                    encoderPosition += 5;
                    if ( encoderPosition > 20)
                        encoderPosition = 20;
                    t_ymove = encoderPosition / 5 - 1;
                    break;
                    
                case 305:
                    encoderPosition += 5;
                    if ( encoderPosition > 20)
                        encoderPosition = 20;
                    t_zmove = encoderPosition / 5 - 1;
                    break;
                    
                case 400: //preheat
                    encoderPosition += 5;
                    if ( encoderPosition > 15)
                        encoderPosition = 15;
                    break;
                    
                case 401:  //nozzle
                    if (target_temperature[0] > 0)
                    {
                        encoderDiff = -1;
                        target_temperature[0]--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0])/*(degTargetHotend(0) + 0.5)*/));
                        enquecommand(cmd);
                        if (target_temperature[0] == 0)
                            HEAT_OFF = true;
                    }
                    break;
                    
                case 402: //bed_temp down
                    if ( target_temperature_bed > 0) 
                    {
                        encoderDiff = -1;
                        target_temperature_bed--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed)/*int(degTargetBed() + 0.5)*/));
                        enquecommand(cmd);
                        if ( target_temperature_bed == 0)
                            BED_OFF = true;
                    }
                    break;
                    
                default:
                    break;
                    
            }

#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_DOWN(0)");
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            //SERIAL_ECHOPGM("sd_file_pos= ");
            //SERIAL_PROTOCOLLN((int)sd_file_pos);
            //SERIAL_ECHOPGM("sd file count= ");
            //SERIAL_PROTOCOLLN(SD_FILE_COUNT);
            SERIAL_ECHOPGM("encoderPosition = ");
            SERIAL_PROTOCOLLN((int)encoderPosition);
            //SERIAL_PROTOCOLLN("encoderDiff= ");
            //SERIAL_PROTOCOLLN((int)encoderDiff);
#endif
/*
            int u;
            if (sd_file_pos > 2)
               u = sd_file_pos - 2;
            else
               u = 0;
            for (int i = 0 ; i < 3; i++)
            {
               card.getfilename(u + i);
               //SERIAL_ECHOPGM("[");
               //SERIAL_PROTOCOL(i+u)
               //SERIAL_ECHOPGM("] = ");
               //SERIAL_PROTOCOLLN(card.filename);
            }
*/
            
        }
        else if (SW_RELEASE == true)
        {
            encoderDiff = 5;
            switch(MENU_PAGE)
            {
                case   1: //lcd_main_menu()
                    encoderPosition += 5;
                    if ( encoderPosition > 25)  //2014/08/25
                        encoderPosition = 25;
                    break;
                    
                case 100:  //print card ok
                    if (SD_FILE_COUNT > 3)
                    {
                        encoderPosition += 5;
                        if ( encoderPosition > 20)
                            encoderPosition = 20;
                        sd_file_pos++;
                        if ( sd_file_pos > SD_FILE_COUNT - 1)
                            sd_file_pos = SD_FILE_COUNT - 1;
                    }
                    else
                    {
                        if (sd_file_pos < SD_FILE_COUNT - 1)
                        {
                            encoderPosition += 5;
                            sd_file_pos++;
                        }
                        if (SD_FILE_COUNT == 0)
                            sd_file_pos = 0;
                    }
                    //encoderPosition += 5;
                    //if ( encoderPosition > 20)
                        //encoderPosition = 20;
                    
                    //sd_file_pos++;
                    //if ( sd_file_pos > SD_FILE_COUNT - 3)
                        //sd_file_pos = SD_FILE_COUNT - 3;
                        
                    //if (sd_file_pos + 1 > SD_FILE_COUNT)
                    //{
                        //sd_file_pos = SD_FILE_COUNT - 1;
                        //encoderPosition = SD_FILE_COUNT * 5;
                    //}
                    break;
                    
                case 101: //print card fail
                    break;
                    
                case 111:
                    if (encoderPosition == 10) 
                        encoderPosition = 15;
                    break;
                    
                case 120: //print adjust
                    encoderPosition += 5;
                    if ( encoderPosition > 30)
                        encoderPosition = 30;
                    break;
                    
                case 121:  //adjust nozzle temp
                    if (target_temperature[0] > 0)
                    {
                        encoderDiff = -1;
                        target_temperature[0]--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0]))); //degTargetHotend(0) + 0.5;
                        enquecommand(cmd);
                    }
                    break;
                    
                case 122:  //adjust bed temp
                    if ( target_temperature_bed > 0) 
                    {
                        encoderDiff = -1;
                        target_temperature_bed--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed)));    //degTargetBed() + 0.5;
                        enquecommand(cmd);
                    }
                    break;
                    
                case 123:  //adjust print speed
                    if (print_speed > 25)
                    {
                        encoderDiff = -1;
                        print_speed--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M220 S%s"), itostr3left(print_speed));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 124:  //adjust gear speed
                    if (gear_speed > 50)
                    {
                        encoderDiff = -1;
                        gear_speed--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M221 S%s"), itostr3left(gear_speed));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 125:  //adjust fan speed
                    if (adjust_fan_speed > 0)
                    {
                        encoderDiff = -1;
                        adjust_fan_speed--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M106 S%s"), itostr3left(int((adjust_fan_speed * 255 ) / 100)));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 126:  //adjust print finish
                    encoderPosition += 5;
                    if ( encoderPosition > 10)
                        encoderPosition = 10;
                    break;
                    
                case 200: //filament mount
                    if (int(degHotend(0) + 0.5) > EXTRUDE_MINTEMP )
                    {
                        char cmd[30], cmd2[10];
                        sprintf_P(cmd, PSTR("G1 E%s F"), itostr3left(FILAMENT_MOVE));
                        sprintf_P(cmd2,PSTR("%s"), itostr3left(FILAMENT_FEEDRATE));
                        for( int i = 0; i < 30; i++)
                        {
                            //SERIAL_ECHOPGM("cmd[] = ");
                            //SERIAL_PROTOCOLLN((int)cmd[i]);
                            if (cmd[i] == 0)
                            {
                                cmd[i] = cmd2[0];
                                cmd[i+1] = cmd2[1];
                                cmd[i+2] = cmd2[2];
                                cmd[i+3] = '\0';
                                break;
                            }
                        }
                        enquecommand_P(PSTR("G91"));
                        enquecommand(cmd);
                        //enquecommand_P(PSTR("G1 E50 F100"));
                        enquecommand_P(PSTR("G90"));
                        //enquecommand_P(PSTR("M114"));
                    }
                    break;
                    
                case 300: //xmove
                    if (t_xmove == 0)
                    {
                        //50mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X-50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_xmove == 1)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X-10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_xmove == 2)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X-1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 301: //ymove
                    if (t_ymove == 0)
                    {
                        //50mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y-50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_ymove == 1)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y-10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_ymove == 2)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y-1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 302: //zmove
                    if (t_zmove == 0)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_zmove == 1)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_zmove == 2)
                    {
                        //0.1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 303:
                    encoderPosition += 5;
                    if ( encoderPosition > 20)
                        encoderPosition = 20;
                    t_xmove = encoderPosition / 5 - 1;
                    break;
                    
                case 304:
                    encoderPosition += 5;
                    if ( encoderPosition > 20)
                        encoderPosition = 20;
                    t_ymove = encoderPosition / 5 - 1;
                    break;
                    
                case 305:
                    encoderPosition += 5;
                    if ( encoderPosition > 20)
                        encoderPosition = 20;
                    t_zmove = encoderPosition / 5 - 1;
                    break;
                    
                case 400: //preheat
                    encoderPosition += 5;
                    if ( encoderPosition > 15)
                        encoderPosition = 15;
                    break;
                    
                case 401:  //nozzle
                    if (target_temperature[0] > 0)
                    {
                        encoderDiff = -1;
                        target_temperature[0]--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0]))); //degTargetHotend(0) + 0.5
                        enquecommand(cmd);
                        if (target_temperature[0] == 0)
                            HEAT_OFF = true;
                    }
                    break;
                    
                case 402: //bed_temp down
                    if ( target_temperature_bed > 0) 
                    {
                        encoderDiff = -1;
                        target_temperature_bed--;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed)));  //degTargetBed() + 0.5
                        enquecommand(cmd);
                        if ( target_temperature_bed == 0)
                            BED_OFF = true;

                    }
                    break;
                    
                default:
                    break;
            }
            
#if SW_MON == 1            
            SERIAL_PROTOCOLLN("SW_DOWN(1)");
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            SERIAL_ECHOPGM("encoderPosition = ");
            SERIAL_PROTOCOLLN((int)encoderPosition);
            //SERIAL_ECHOPGM("encoderDiff= ");
            //SERIAL_PROTOCOLLN((int)encoderDiff);
            //SERIAL_ECHOPGM("sd_file_pos= ");
            //SERIAL_PROTOCOLLN((int)sd_file_pos);
            //SERIAL_ECHOPGM("sd file count= ");
            //SERIAL_PROTOCOLLN(SD_FILE_COUNT);
#endif

            //int u;
            //if (sd_file_pos > 2)
               //u = sd_file_pos - 2;
            //else
               //u = 0;
            //for (int i = 0 ; i < 3; i++)
            //{
                //card.getfilename(u + i);
                //SERIAL_ECHOPGM("[");
                //SERIAL_PROTOCOL(i+u)
                //SERIAL_ECHOPGM("] = ");
                //SERIAL_PROTOCOLLN(card.filename);
            //}
            
            newbutton |= EN_A;
            SW_RELEASE = false;
        }
    }
    
    //UP SW
    if ( (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 0) && (READ(BTN_LEFT) == 1) && (READ(BTN_RIGHT) == 1) )
    
    //if(/*(blocking_enc<millis()) &&*/ (READ(BTN_EN2) == 0))
    {
        delay(2);
        if (READ(BTN_EN2) != 0)
            return;
        
        if (PUSH_SW != 2) //up
        {
            if ((encoderPosition - (int)(encoderPosition / 5) * 5) != 0 )
            //if (encoderPosition == 19) 
                encoderPosition += 1;
            
            
            SW_RELEASE = false; 
            newbutton |= EN_B;
            PUSH_SW = 2;
            
            mente[mente_no++] = 3;
            
            switch(MENU_PAGE)
            {
                case 100:  //print card ok
                    encoderDiff = -5;
                    if (encoderPosition > 0)
                        encoderPosition -= 5;
                        
                    sd_file_pos--;
                    if ( sd_file_pos < 0)
                        sd_file_pos = 0;
                    break;
                    
                case 101: //print card fail
                    break;
                   
                case 111:
                     if (encoderPosition == 15) 
                         encoderPosition = 10;
                     break;
                     
                case 121:  //adjust nozzle temp
                    if (target_temperature[0] < HEAT_TEMP_MAX)
                    {
                        encoderDiff = 1;
                        target_temperature[0]++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0]/*degTargetHotend(0) + 0.5*/)));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 122:  //adjust bed temp
                    if ( target_temperature_bed < BED_TEMP_MAX) 
                    {
                        encoderDiff = 1;
                        target_temperature_bed++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed/*degTargetBed() + 0.5*/)));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 123:  //adjust print speed
                    if (print_speed < PRINT_SPEED_MAX)
                    {
                        encoderDiff = 1;
                        print_speed++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M220 S%s"), itostr3left(print_speed));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 124:  //adjust gear speed
                    if (gear_speed < 150)
                    {
                        encoderDiff = 1;
                        gear_speed++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M221 S%s"), itostr3left(gear_speed));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 125:  //adjust fan speed
                    if (adjust_fan_speed < 100)
                    {
                        encoderDiff = 1;
                        adjust_fan_speed++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M106 S%s"), itostr3left(int((adjust_fan_speed * 255 ) / 100)));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 126:  //adjust print finish
                    encoderDiff = -5;
                    if (encoderPosition > 0)
                        encoderPosition -= 5;
                    break;
                    
                case 200: //filament remove
                    if (int(degHotend(0) + 0.5) > EXTRUDE_MINTEMP )
                    {
                        char cmd[30], cmd2[10];
                        sprintf_P(cmd, PSTR("G1 E-%s F"), itostr3left(FILAMENT_MOVE));
                        sprintf_P(cmd2,PSTR("%s"), itostr3left(FILAMENT_FEEDRATE));
                        for( int i = 0; i < 30; i++)
                        {
                            //SERIAL_ECHOPGM("cmd[] = ");
                            //SERIAL_PROTOCOLLN((int)cmd[i]);
                            if (cmd[i] == 0)
                            {
                                cmd[i] = cmd2[0];
                                cmd[i+1] = cmd2[1];
                                cmd[i+2] = cmd2[2];
                                cmd[i+3] = '\0';
                                break;
                            }
                        }
                        enquecommand_P(PSTR("G91"));
                        enquecommand(cmd);
                        //enquecommand_P(PSTR("G1 E50 F100"));
                        enquecommand_P(PSTR("G90"));
                        //enquecommand_P(PSTR("M114"));
                    }
                    break;
                    
                case 300: //xmove
                    if (t_xmove == 0)
                    {
                        //50mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_xmove == 1)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_xmove == 2)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 301: //ymove
                    if (t_ymove == 0)
                    {
                        //50mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_ymove == 1)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_ymove == 2)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 302: //zmove
                    if (t_zmove == 0)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z-50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_zmove == 1)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z-10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_zmove == 2)
                    {
                        //0.1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z-1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 303:
                    encoderDiff = -5;
                    if (encoderPosition > 5)
                        encoderPosition -= 5;
                    t_xmove = encoderPosition / 5 - 1;
                    break;
                    
                case 304:
                    encoderDiff = -5;
                    if (encoderPosition > 5)
                        encoderPosition -= 5;
                    t_ymove = encoderPosition / 5 - 1;
                    break;
                    
                case 305:
                    encoderDiff = -5;
                    if (encoderPosition > 5)
                        encoderPosition -= 5;
                    t_zmove = encoderPosition / 5 - 1;
                    break;
                    
                case 401: //nozzle_temp up
                    if (HEAT_TEMP_MAX > target_temperature[0]) 
                    {
                        encoderDiff = 1;
                        target_temperature[0]++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0]/*degTargetHotend(0) + 0.5*/)));
                        enquecommand(cmd);
                        HEAT_OFF = false;
                    }
                    break;
                     
                case 402: //bed_temp up
                    if (BED_TEMP_MAX > target_temperature[0]) 
                    {
                        encoderDiff = 1;
                        target_temperature_bed++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed/*degTargetBed() + 0.5*/)));
                        enquecommand(cmd);
                        BED_OFF = false;
                    }
                    break;
                    
                default:
                    encoderDiff = -5;
                    if (encoderPosition > 0)
                        encoderPosition -= 5;
                    break;
                    
            }
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_UP(0)");
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            SERIAL_ECHOPGM("encoderPosition = ");
            SERIAL_PROTOCOLLN((int)encoderPosition);
            //SERIAL_ECHOPGM("sd_file_pos= ");
            //SERIAL_PROTOCOLLN((int)sd_file_pos);
            //SERIAL_ECHOPGM("sd file count= ");
            //SERIAL_PROTOCOLLN(SD_FILE_COUNT);
            //SERIAL_PROTOCOLLN("encoderDiff= ");
            //SERIAL_PROTOCOLLN((int)encoderDiff);
#endif
        }
        else if (SW_RELEASE == true)
        {
            switch(MENU_PAGE)
            {
                case 100:  //print card ok
                    encoderDiff = -5;
                    if (encoderPosition > 0)
                        encoderPosition -= 5;
                        
                    sd_file_pos--;
                    if ( sd_file_pos < 0)
                        sd_file_pos = 0;
                    break;
                   
                case 101: //print card fail
                    break;
                   
                case 111:
                     if (encoderPosition == 15) 
                         encoderPosition = 10;
                     break;
                     
                case 121:  //adjust nozzle temp
                    if (target_temperature[0] < HEAT_TEMP_MAX)
                    {
                        encoderDiff = 1;
                        target_temperature[0]++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0]/*degTargetHotend(0) + 0.5*/)));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 122:  //adjust bed temp
                    if ( target_temperature_bed < BED_TEMP_MAX) 
                    {
                        encoderDiff = 1;
                        target_temperature_bed++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed/*degTargetBed() + 0.5*/)));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 123:  //adjust print speed
                    if (print_speed < PRINT_SPEED_MAX)
                    {
                        encoderDiff = 1;
                        print_speed++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M220 S%s"), itostr3left(print_speed));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 124:  //adjust gear speed
                    if (gear_speed < 150)
                    {
                        encoderDiff = 1;
                        gear_speed++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M221 S%s"), itostr3left(gear_speed));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 125:  //adjust fan speed
                    if (adjust_fan_speed < 100)
                    {
                        encoderDiff = 1;
                        adjust_fan_speed++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M106 S%s"), itostr3left(int((adjust_fan_speed * 255 ) / 100)));
                        enquecommand(cmd);
                    }
                    break;
                    
                case 126:  //adjust print finish
                    encoderDiff = -5;
                    if (encoderPosition > 0)
                        encoderPosition -= 5;
                    break;
                    
                case 200: //filament remove
                   if (int(degHotend(0) + 0.5) > EXTRUDE_MINTEMP )
                   {
                       char cmd[30], cmd2[10];
                       sprintf_P(cmd, PSTR("G1 E-%s F"), itostr3left(FILAMENT_MOVE));
                       sprintf_P(cmd2,PSTR("%s"), itostr3left(FILAMENT_FEEDRATE));
                       for( int i = 0; i < 30; i++)
                       {
                           //SERIAL_ECHOPGM("cmd[] = ");
                           //SERIAL_PROTOCOLLN((int)cmd[i]);
                           if (cmd[i] == 0)
                           {
                               cmd[i] = cmd2[0];
                               cmd[i+1] = cmd2[1];
                               cmd[i+2] = cmd2[2];
                               cmd[i+3] = '\0';
                               break;
                           }
                       }
                       enquecommand_P(PSTR("G91"));
                       enquecommand(cmd);
                       //enquecommand_P(PSTR("G1 E50 F100"));
                       enquecommand_P(PSTR("G90"));
                       //enquecommand_P(PSTR("M114"));
                   }
                   break;
                   
                case 300: //xmove
                    if (t_xmove == 0)
                    {
                        //50mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_xmove == 1)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_xmove == 2)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 X1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 301: //ymove
                    if (t_ymove == 0)
                    {
                        //50mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_ymove == 1)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_ymove == 2)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Y1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 302: //zmove
                    if (t_zmove == 0)
                    {
                        //10mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z-50 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_zmove == 1)
                    {
                        //1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z-10 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    if (t_zmove == 2)
                    {
                        //0.1mm
                        enquecommand_P(PSTR("G91"));
                        enquecommand_P(PSTR("G950 Z-1 F4800"));
                        enquecommand_P(PSTR("G90"));
                    }
                    break;
                    
                case 303:
                    encoderDiff = -5;
                    if (encoderPosition > 5)
                        encoderPosition -= 5;
                    t_xmove = encoderPosition / 5 - 1;
                    break;
                    
                case 304:
                    encoderDiff = -5;
                    if (encoderPosition > 5)
                        encoderPosition -= 5;
                    t_ymove = encoderPosition / 5 - 1;
                    break;
                    
                case 305:
                    encoderDiff = -5;
                    if (encoderPosition > 5)
                        encoderPosition -= 5;
                    t_zmove = encoderPosition / 5 - 1;
                    break;
                    
                case 350:
                    encoderDiff = -1;
                    //if (encoderPosition > 0)
                    encoderPosition--;
                    //if (encoderPosition < 0) encoderPosition = 0;
                    break;
                    
                case 401: //nozzle_temp up
                    if ((HEAT_TEMP_MAX ) > target_temperature[0]) 
                    {
                        encoderDiff = 1;
                        target_temperature[0]++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M104 S%s"), itostr3left((int)(target_temperature[0]/*degTargetHotend(0) + 0.5*/)));
                        enquecommand(cmd);
                        HEAT_OFF = false;
                    }
                    break;
                    
                case 402: //bed_temp up
                    if (BED_TEMP_MAX > target_temperature_bed) 
                    {
                        encoderDiff = 1;
                        target_temperature_bed++;
                        char cmd[30];
                        sprintf_P(cmd, PSTR("M140 S%s"), itostr3left(int(target_temperature_bed/*degTargetBed() + 0.5*/)));
                        enquecommand(cmd);
                        BED_OFF = false;
                    }
                    break;
                    
                default:
                    encoderDiff = -5;
                    if (encoderPosition > 0)
                        encoderPosition -= 5;
                    break;
            }
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_UP(1)");
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            SERIAL_ECHOPGM("encoderPosition = ");
            SERIAL_PROTOCOLLN((int)encoderPosition);
            //SERIAL_ECHOPGM("sd_file_pos= ");
            //SERIAL_PROTOCOLLN((int)sd_file_pos);
            //SERIAL_ECHOPGM("sd file count= ");
            //SERIAL_PROTOCOLLN(SD_FILE_COUNT);
            //SERIAL_PROTOCOLLN("encoderDiff= ");
            //SERIAL_PROTOCOLLN((int)encoderDiff);
#endif

            newbutton |= EN_B;
            SW_RELEASE = false;
        }
    }
/*    
    if(READ(BTN_LEFT) == 0)
    {
        delay(2);
        if (READ(BTN_LEFT) != 0)
            return;

        if (PUSH_SW != 3)
        {
            SW_RELEASE = false;
            PUSH_SW = 3; SERIAL_PROTOCOLLN("SW_LEFT(0)");
            
            if (MENU_PAGE == 1)
                return;
            
            switch (MENU_PAGE)
            {
                case 100:  //print card ok
                case 101:  //print card fail
                    sd_print_flg = false;
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    encoderPosition = 5;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                
                case 200:  //Filament
                    pre_heat_flg = false;
                    enquecommand_P(PSTR("M104 S0"));  //extruder heat off
                    encoderPosition = 10;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                
                case 300:  //control
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 400:  //preheat
                    lcd_cls_flg = false;
                    encoderPosition = 20;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 401:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                case 402:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 10;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                default:
                    break;
            }
            lcdDrawUpdate = 2;
            //lcd_next_update_millis = millis() + 100;
            
            
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            //SERIAL_PROTOCOLPGM("encoderPosition =");
            //SERIAL_PROTOCOLLN((int)encoderPosition);
        }
        else if (SW_RELEASE == true)
        {
            SERIAL_PROTOCOLLN("SW_LEFT(1)");
            
            if (MENU_PAGE == 1)
                return;
            
            switch (MENU_PAGE)
            {
                case 100:  //print card ok
                case 101:  //print card fail
                    sd_print_flg = false;
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    encoderPosition = 5;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                
                case 200:  //Filament
                    pre_heat_flg = false;
                    enquecommand_P(PSTR("M104 S0"));  //extruder heat off
                    encoderPosition = 10;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                
                case 300:  //control
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                
                case 400:  //preheat
                    lcd_cls_flg = false;
                    encoderPosition = 20;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 401:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                case 402:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 10;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                default:
                    break;
            }
            lcdDrawUpdate = 2;
            //lcd_next_update_millis = millis() + 100;
            
            
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            //SERIAL_PROTOCOLPGM("encoderPosition =");
            //SERIAL_PROTOCOLLN((int)encoderPosition);
        }
    }
*/

    //ENTER SW
    if( (blocking_enc<millis()) && (READ(BTN_ENC)== 0)) 
    //if(READ(BTN_ENC) == 0)
    {
        //delay(2);
        //if (READ(BTN_ENC) != 0)
        //   return;
        
        if (PUSH_SW != 5) 
        {
            SW_RELEASE = false;
            encoderDiff = -1;
            bak_encpos = encoderPosition;
            
            PUSH_SW = 5;
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_ENTER(0)");
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            SERIAL_PROTOCOLPGM("encoderPosition =");
            SERIAL_PROTOCOLLN((int)encoderPosition);
            //SERIAL_PROTOCOLPGM("bak_encpos =");
            //SERIAL_PROTOCOLLN((int)bak_encpos);
            //SERIAL_PROTOCOLPGM("encoderDiff= ");
            //SERIAL_PROTOCOLLN((int)encoderDiff);
#endif
            
            /*
            //SD print
            if ((MENU_PAGE == 100) && (sd_print_flg  == false))
            {
                sd_print_flg  = true;
                lcd_cls_flg = false;
                
                SERIAL_PROTOCOLLN("PRINT GOGO(1) ");
                card.getfilename(sd_file_pos);
                
                //card.getWorkDirName();
                if (card.filenameIsDir)
                {
                    //folder
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    
                    //SERIAL_PROTOCOLPGM("ENTER::FOLDER = ");
                    //SERIAL_PROTOCOLLN(card.filename);
                    card.chdir(card.filename);
                    card.getWorkDirName();
                    //SERIAL_PROTOCOLPGM("Workdir = ");
                    //SERIAL_PROTOCOLLN(card.getWorkDirName());
                    
                    lcd_quick_feedback();
                    currentMenu = lcd_sdcard_menu;
                }
                else
                {
                    if (Fcnt == 0)
                    {
                        lcd_quick_feedback();
                        currentMenu = lcd_sdcard_menu;
                    }
                    else
                    {
                        char cmd[30];
                        char* c;
                        sprintf_P(cmd, PSTR("M23 %s"), card.filename);
                        for(c = &cmd[4]; *c; c++)
                            *c = tolower(*c);
                        enquecommand(cmd);
                        //enquecommand_P(PSTR("M17"));  //stepper all on
                        
                        enquecommand_P(PSTR("M24"));
                        
                        //delay(20);  //open٤
                        //SERIAL_PROTOCOLPGM("PRINT IS_SD_PRINTING =  ");
                        //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                        //enquecommand_P(PSTR("M27"));
                        
                        lcd_quick_feedback();
                        currentMenu = lcd_print_screen;
                    }
                }
            }
            */
            
            
            if (MENU_PAGE == 1)
            {
                if (encoderPosition == 5)
                {
                    no_card_flg == false;
                    lcd_cls_flg = false;
                    RC = 0;
                    lcd_quick_feedback();
                    currentMenu = lcd_sdcard_menu;
                }
            }
            
            if (MENU_PAGE == 101)
            {
                //SERIAL_PROTOCOLLN("ENTER SW = 101 ret");
                return;
            }
            
            /*
            //2014/08/25
            if (MENU_PAGE == 101)
            {
                if (encoderPosition == 5)
                {
                    no_card_flg == false;
                    lcd_cls_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    //SERIAL_PROTOCOLLN("PAGE(0) = 101 ");
                    //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                }
            }
            ///
            */
            
            /*
            
            if (MENU_PAGE == 110)  //adjust
            {
                SERIAL_PROTOCOLPGM("Adjust(0) ");
            }
            
            if (MENU_PAGE == 300)
            {
                lcd_quick_feedback();
                currentMenu = lcd_xmove50;
            }
            if (MENU_PAGE == 301)
            {
                lcd_quick_feedback();
                currentMenu = lcd_ymove50;
            }
            if (MENU_PAGE == 302)
            {
                lcd_quick_feedback();
                currentMenu = lcd_zmove50;
            }
*/

/*
            if (MENU_PAGE == 111)
            {
                if (encoderPosition == 5)
                {
                    //No
                    SERIAL_PROTOCOLLN("PRINT END(0) - TOP  ");
                    lcd_cls_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                }
                else
                {
                    //Yes
                    sd_print_flg  = true;
                    lcd_cls_flg = false;
                    
                    SERIAL_PROTOCOLLN("REPRINT  ");
                    card.getfilename(sd_file_pos);
                    
                    char cmd[30];
                    char* c;
                    sprintf_P(cmd, PSTR("M23 %s"), card.filename);
                    for(c = &cmd[4]; *c; c++)
                        *c = tolower(*c);
                    enquecommand(cmd);
                    //delay(20);
                    enquecommand_P(PSTR("M24"));
                    lcd_quick_feedback();
                    currentMenu = lcd_print_screen;
                }
            }
*/
            //adjust
            if (MENU_PAGE == 110)
            {
                //PUSH_SW = 0;
                //adjust_flg = true;
                //lcd_quick_feedback();
                currentMenu = lcd_print_adjust_menu;
            }
            
            
            //x move
            if (MENU_PAGE == 303)
            {
                if (t_xmove != 3)
                {
                    xcmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                }
                else
                {
                    if (xcmd_flg == false)
                    {
                        xcmd_flg = true;
                        enquecommand_P(PSTR("G28 X0"));
                    }
                }
            }
            //y move
            if (MENU_PAGE == 304)
            {
                if (t_ymove != 3)
                {
                    ycmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_y_menu;
                }
                else
                {
                    if (ycmd_flg == false)
                    {
                        ycmd_flg = true;
                        enquecommand_P(PSTR("G28 X0"));
                    }
                }
            }
            //z move
            if (MENU_PAGE == 305)
            {
                if (t_zmove != 3)
                {
                    zcmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                }
                else
                {
                    if (zcmd_flg == false)
                    {
                        zcmd_flg = true;
                        enquecommand_P(PSTR("G28 Z0"));
                    }
                }
            }
        }
        
        else if (SW_RELEASE == true)
        {
            //SERIAL_PROTOCOLLN("SW_ENTER(1)");
            //SERIAL_ECHOPGM("page = ");
            //SERIAL_PROTOCOLLN((int)MENU_PAGE);
            //SERIAL_PROTOCOLPGM("encoderPosition =");
            //SERIAL_PROTOCOLLN((int)encoderPosition);
            
            SW_RELEASE = false;
            
            //encoderDiff = -1;
            bak_encpos = encoderPosition;
            
            //SD print
            if ((MENU_PAGE == 100) && (sd_print_flg  == false))
            {
                sd_print_flg  = true;
                lcd_cls_flg = false;
                FINISH_PROC = false;
                
                //SERIAL_PROTOCOLLN("PRINT GOGO(1) ");
                card.getfilename(sd_file_pos);
                
                //card.getWorkDirName();
                if (card.filenameIsDir)
                {
                    //folder
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    
                    //SERIAL_PROTOCOLPGM("ENTER::FOLDER = ");
                    //SERIAL_PROTOCOLLN(card.filename);
                    card.chdir(card.filename);
                    card.getWorkDirName();
                    //SERIAL_PROTOCOLPGM("Workdir = ");
                    //SERIAL_PROTOCOLLN(card.getWorkDirName());
                    RC = 0;
                    lcd_quick_feedback();
                    currentMenu = lcd_sdcard_menu;
                }
                else
                {
                    if (Fcnt == 0)
                    {
                        RC = 0;
                        lcd_quick_feedback();
                        currentMenu = lcd_sdcard_menu;
                    }
                    else
                    {
                        char cmd[30];
                        char* c;
                        sprintf_P(cmd, PSTR("M23 %s"), card.filename);
                        for(c = &cmd[4]; *c; c++)
                            *c = tolower(*c);
                        enquecommand(cmd);
                        //enquecommand_P(PSTR("M17"));  //stepper all on
                        
                        enquecommand_P(PSTR("M24"));
                        
                        //delay(20);  //open٤
                        //SERIAL_PROTOCOLPGM("PRINT IS_SD_PRINTING =  ");
                        //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                        //enquecommand_P(PSTR("M27"));
                        
                        lcd_quick_feedback();
                        currentMenu = lcd_print_screen;
                    }
                }
            }
            
            if (MENU_PAGE == 101)
            {
                //SERIAL_PROTOCOLLN("ENTER SW = 101 ret");
                return;
            }
            
            //2014/08/25
            /*
            if (MENU_PAGE == 101)
            {
                if (encoderPosition == 5)
                {
                    no_card_flg == false;
                    lcd_cls_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    //SERIAL_PROTOCOLLN("PAGE(1) = 101 ");
                }
            }
            ///
            */
            
            
            //adjust
            if (MENU_PAGE == 110)
            {
                PUSH_SW = 0;
                encoderPosition = 5;
                //adjust_flg = true;
                lcd_quick_feedback();
                currentMenu = lcd_print_adjust_menu;
            }
            
            //print finish
            if (MENU_PAGE == 111)
            {
                if (encoderPosition == 10)
                {
                    //No
                    SERIAL_PROTOCOLLN("PRINT END(1) - TOP  ");
                    encoderPosition = 5;
                    lcd_cls_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_finish_proc;  // cd_main_menu;
                }
                else if ((encoderPosition == 15) && (sd_print_flg == false))
                {
                    //Yes
                    sd_print_flg  = true;
                    lcd_cls_flg = false;
                    
                    //SERIAL_PROTOCOLLN("RE-PRINT  ");
                    card.getfilename(sd_file_pos);
                    
                    char cmd[30];
                    char* c;
                    sprintf_P(cmd, PSTR("M23 %s"), card.filename);
                    for(c = &cmd[4]; *c; c++)
                        *c = tolower(*c);
                    enquecommand(cmd);
                    enquecommand_P(PSTR("M17"));

                    //delay(20);  //open٤
                    enquecommand_P(PSTR("M24"));
                    
                    //SERIAL_PROTOCOLPGM("PRINT IS_SD_PRINTING =  ");
                    //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                    
                    lcd_quick_feedback();
                    currentMenu = lcd_print_screen;
                }
            }
            
            //adjust nozzle
            if (MENU_PAGE == 120)
            {
                if (encoderPosition == 5)
                {
                    //nozzle
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_nozzle;
                }
                if (encoderPosition == 10)
                {
                    //bed temp
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_bed_temp;
                }
                if (encoderPosition == 15)
                {
                    //print speed
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_print_speed;
                }
                if (encoderPosition == 20)
                {
                    //gear speed
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_gear_speed;
                }
                if (encoderPosition == 25)
                {
                    //fan speed
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_fan_speed;
                }
                if (encoderPosition == 30)
                {
                    //print fin.
                    encoderPosition = 5;
                    lcd_cls_flg = false;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_print_finish;
                }
            }
            
            //kill print
            if (MENU_PAGE == 126) //print fin.
            {
                //if (IS_SD_PRINTING == false)
                //    return;
                
                if (encoderPosition == 5)
                {
                    lcd_quick_feedback();
                    currentMenu = lcd_print_screen;
                }
                else
                {
                    //if (IS_SD_PRINTING == true)
                    {
                        //λ
                        //lcd_sdcard_stop();
                        
                        //SERIAL_PROTOCOLPGM("PRINT IS_SD_PRINTING =  ");
                        //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                        
                        sd_print_flg = false;
                        exec_print = false;
                        card.sdprinting = false;
                        
                        PUSH_SW = 0;
                        encoderPosition = 5;
                        //lcdDrawUpdate = 2;
                        lcd_cls_flg = false;
                        lcd_quick_feedback();
                        currentMenu = lcd_print_finish_proc;
                        
                        //card.cardOK = false;
                        //st_synchronize();
                        //quickStop();
                        //card.closefile();
                        
                        //enquecommand_P(PSTR("M22"));   //release SD
                        //enquecommand_P(PSTR("M84"));   //all stepper disable
                        
                        //if(SD_FINISHED_STEPPERRELEASE)
                        //{
                        //    enquecommand_P(PSTR(SD_FINISHED_RELEASECOMMAND));
                        //}
                        //autotempShutdown();
                        
                        /*
                        for (;;)
                        {
                            //if (card.cardOK == false)
                            {
                                //SERIAL_PROTOCOLPGM("PRINT IS_SD_PRINTING =  ");
                                //SERIAL_PROTOCOLLN(IS_SD_PRINTING);
                                SERIAL_PROTOCOLPGM("SD status = ");
                                enquecommand_P(PSTR("M27"));
                                break;
                            }
                        }
                        */
                        
                        
                    }
                }
            }
            
            if (MENU_PAGE == 300)
            {
                lcd_quick_feedback();
                currentMenu = lcd_xmove50;
            }
            if (MENU_PAGE == 301)
            {
                lcd_quick_feedback();
                currentMenu = lcd_ymove50;
            }
            if (MENU_PAGE == 302)
            {
                lcd_quick_feedback();
                currentMenu = lcd_zmove50;
            }
            if (MENU_PAGE == 303)
            {
                if (t_xmove != 3)
                {
                    xcmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                }
                else
                {
                    if (xcmd_flg == false)
                    {
                        xcmd_flg = true;
                        enquecommand_P(PSTR("G28 X0"));
                    }
                }
            }
            if (MENU_PAGE == 304)
            {
                if (t_ymove != 3)
                {
                    ycmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_y_menu;
                }
                else
                {
                    if (ycmd_flg == false)
                    {
                        ycmd_flg = true;
                        enquecommand_P(PSTR("G28 X0"));
                    }
                }
            }
            if (MENU_PAGE == 305)
            {
                if (t_zmove != 3)
                {
                    zcmd_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                }
                else
                {
                    if (zcmd_flg == false)
                    {
                        zcmd_flg = true;
                        enquecommand_P(PSTR("G28 Z0"));
                    }
                }
            }
        }
        
        newbutton |= EN_C;
    }
    buttons = newbutton;
    
    //LEFT SW
    if ( (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_LEFT) == 0) && (READ(BTN_RIGHT) == 1) )
    //if(READ(BTN_LEFT) == 0)
    {
        delay(2);
        if (READ(BTN_LEFT) != 0)
            return;

        if (PUSH_SW != 3)
        {
            SW_RELEASE = false;
            PUSH_SW = 3; 
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_LEFT(0)");
#endif
            
            mente[mente_no++] = 2;
            
            if (MENU_PAGE == 1)
                return;
            
            switch (MENU_PAGE)
            {
                case 100:  //print card ok
                    sd_print_flg = false;
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    card.setroot();
                    
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 101:  //print card fail
                    sd_print_flg = false;
                    no_card_flg = false;    //2014/08/25
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    card.setroot();
                    
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 120:
                    lcd_cls_flg = false;
                    encoderPosition = 5;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_screen;
                    break;
                    
                case 121:  //print adjust nozzle
                    lcd_cls_flg = false;
                    encoderPosition = 5;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 122:  //print adjust bed
                    lcd_cls_flg = false;
                    encoderPosition = 10;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 123:  //print adjust print speed
                    lcd_cls_flg = false;
                    encoderPosition = 15;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 124:  //print adjust gear speed
                    lcd_cls_flg = false;
                    encoderPosition = 20;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 125:  //print adjust fan speed
                    lcd_cls_flg = false;
                    encoderPosition = 25;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 126:  //print adjust print speed
                    lcd_cls_flg = false;
                    encoderPosition = 30;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 200:  //Filament
                    pre_heat_flg = false;
                    enquecommand_P(PSTR("M104 S0"));  //extruder heat off
                    encoderPosition = 10;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 300:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                    break;
                    
                case 301:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                    break;
                    
                case 302:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 303:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                    break;
                    
                case 304:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_y_menu;
                    break;
                    
                case 305:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                    break;
                    
                case 400:  //preheat
                    lcd_cls_flg = false;
                    encoderPosition = 20;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 401:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                case 402:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 10;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                case 500:  //temp monitor
                    lcd_cls_flg = false;
                    encoderPosition = 25;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                default:
                    break;
            }
            //lcdDrawUpdate = 2;
            //lcd_next_update_millis = millis() + 100;
            
#if SW_MON == 1
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            //SERIAL_PROTOCOLPGM("encoderPosition =");
            //SERIAL_PROTOCOLLN((int)encoderPosition);
#endif
        }
        else if (SW_RELEASE == true)
        {
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_LEFT(1)");
#endif
            
            SW_RELEASE = false;
            
            if (MENU_PAGE == 1)
                return;
            
            switch (MENU_PAGE)
            {
                case 100:  //print card ok
                    sd_print_flg = false;
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    card.setroot();
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 101:  //print card fail
                    sd_print_flg = false;
                    no_card_flg = false;    //2014/08/25
                    sd_file_pos = 0;
                    sd_file_pos_bak = -1;
                    card.setroot();
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 120:
                    lcd_cls_flg = false;
                    encoderPosition = 5;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_screen;
                    break;
                    
                case 121:  //print adjust nozzle
                    lcd_cls_flg = false;
                    encoderPosition = 5;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 122:  //print adjust bed
                    lcd_cls_flg = false;
                    encoderPosition = 10;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 123:  //print adjust print speed
                    lcd_cls_flg = false;
                    encoderPosition = 15;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 124:  //print adjust gear speed
                    lcd_cls_flg = false;
                    encoderPosition = 20;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 125:  //print adjust fan speed
                    lcd_cls_flg = false;
                    encoderPosition = 25;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 126:  //print adjust print speed
                    lcd_cls_flg = false;
                    encoderPosition = 30;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_print_adjust_menu;
                    break;
                    
                case 200:  //Filament
                    pre_heat_flg = false;
                    enquecommand_P(PSTR("M104 S0"));  //extruder heat off
                    encoderPosition = 10;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 300:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                    break;
                    
                case 301:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                    break;
                    
                case 302:
                    lcd_cls_flg = false;
                    encoderPosition = 15;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 303:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_x_menu;
                    break;
                    
                case 304:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_y_menu;
                    break;
                    
                case 305:
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                    break;
                    
                case 400:  //preheat
                    lcd_cls_flg = false;
                    encoderPosition = 20;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                case 401:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 5;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                case 402:  //nozzle heating
                    lcd_cls_flg = false;
                    encoderPosition = 10;//bak_encpos;
                    //lcdDrawUpdate = 2;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                    break;
                    
                case 500:  //temp monitor
                    lcd_cls_flg = false;
                    encoderPosition = 25;//bak_encpos;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    break;
                    
                default:
                    break;
            }
            //lcdDrawUpdate = 2;
            //lcd_next_update_millis = millis() + 100;
            
            
#if SW_MON == 1
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
            //SERIAL_PROTOCOLPGM("encoderPosition =");
            //SERIAL_PROTOCOLLN((int)encoderPosition);
#endif
        }
    }
    
    //RIGHT SW
    if ( (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1) && (READ(BTN_LEFT) == 1) && (READ(BTN_RIGHT) == 0) )
    //if(READ(BTN_RIGHT) == 0)
    {
        delay(2);
        if (READ(BTN_RIGHT) != 0)
            return;
        
        if (PUSH_SW != 4)
        {
            SW_RELEASE = false;
            PUSH_SW = 4;
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_RIGHT(0)");
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
#endif
            
            mente[mente_no++] = 1;
            
            if (MENU_PAGE == 1)
            {
                if (encoderPosition == 0)
                {
                    if (READ(SDCARDDETECT) != 0)
                        card.cardOK = false;
                    
                    R_SW_CNT = 0;
                    RC = 0;
                    encoderPosition = 5;
                    sd_print_flg = false;
                    sd_file_pos_bak = -1;
                    sd_file_pos = 0;
                    lcd_quick_feedback();
                    currentMenu = lcd_sdcard_menu;
                }
                if (encoderPosition == 5)
                {
                    //SERIAL_PROTOCOLLN("R-SW");
                    encoderDiff = 5;
                    R_SW_CNT = 0;
                    RC = 0;
                    encoderPosition = 5;
                    sd_print_flg = false;
                    sd_file_pos_bak = -1;
                    sd_file_pos = 0;
                    lcd_quick_feedback();
                    currentMenu = lcd_sdcard_menu;
                }
                if (encoderPosition == 10)
                {
                    encoderPosition = 5;
                    lcd_quick_feedback();
                    currentMenu = lcd_filament_heating;
                }
                if (encoderPosition == 15)
                {
                    encoderPosition = 5;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                }
                if (encoderPosition == 20)
                {
                    encoderPosition = 5;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                }
                //2014/08/25
                if (encoderPosition == 25)
                {
                    encoderPosition = 5;
                    lcd_quick_feedback();
                    currentMenu = lcd_temp_monitor;
                }
            }
            
            if (MENU_PAGE == 101)
            {
                //SERIAL_PROTOCOLLN("RIGHT_SW(1) = 101 ret");
                return;
            }
            
            /*
            //2014/08/25
            if (MENU_PAGE == 101)
            {
                if (encoderPosition == 5)
                {
                    no_card_flg == false;
                    lcd_cls_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    //SERIAL_PROTOCOLLN("PAGE(1) = 101 ");
                }
            }
            */
            ///
            
            if (MENU_PAGE == 300)
            {
                lcd_cls_flg = false;
                encoderPosition = 5;
                //lcdDrawUpdate = 2;
                lcd_quick_feedback();
                currentMenu = lcd_move_y_menu;
            }
            if (MENU_PAGE == 301)
            {
                lcd_cls_flg = false;
                encoderPosition = 15;
                //lcdDrawUpdate = 2;
                lcd_quick_feedback();
                currentMenu = lcd_main_menu;
            }
            if (MENU_PAGE == 302)
            {
                lcd_cls_flg = false;
                encoderPosition = 15;
                //lcdDrawUpdate = 2;
                lcd_quick_feedback();
                currentMenu = lcd_move_x_menu;
            }
        }
        else if (SW_RELEASE == true)
        {
            delay(2);
            if (READ(BTN_RIGHT) != 0)
                return;
            
            SW_RELEASE = false;
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_RIGHT(1)");
#endif
            
            if (MENU_PAGE == 1)
            {
                if (encoderPosition == 0)
                {
                    if (READ(SDCARDDETECT) != 0)
                        card.cardOK = false;
                    
                    R_SW_CNT = 0;
                    RC = 0;
                    encoderPosition = 5;
                    sd_print_flg = false;
                    sd_file_pos_bak = -1;
                    sd_file_pos = 0;
                    lcd_quick_feedback();
                    currentMenu = lcd_sdcard_menu;
                }
                if (encoderPosition == 5)
                {
                    R_SW_CNT = 0;
                    RC = 0;
                    encoderPosition = 5;
                    sd_print_flg = false;
                    sd_file_pos_bak = -1;
                    sd_file_pos = 0;
                    lcd_quick_feedback();
                    currentMenu = lcd_sdcard_menu;
                }
                if (encoderPosition == 10)
                {
                    encoderPosition = 5;
                    lcd_quick_feedback();
                    currentMenu = lcd_filament_heating;
                }
                if (encoderPosition == 15)
                {
                    encoderPosition = 5;
                    lcd_quick_feedback();
                    currentMenu = lcd_move_z_menu;
                }
                if (encoderPosition == 20)
                {
                    encoderPosition = 5;
                    lcd_quick_feedback();
                    currentMenu = lcd_preheat_menu;
                }
                //2014/08/25
                if (encoderPosition == 25)
                {
                    encoderPosition = 5;
                    lcd_quick_feedback();
                    currentMenu = lcd_temp_monitor;
                }
            }
            
            if (MENU_PAGE == 101)
            {
                //SERIAL_PROTOCOLLN("RIGHT_SW(1) = 101 ret");
                return;
            }
            
            /*
            //2014/08/25
            if (MENU_PAGE == 101)
            {
                if (encoderPosition == 5)
                {
                    no_card_flg == false;
                    lcd_cls_flg = false;
                    lcd_quick_feedback();
                    currentMenu = lcd_main_menu;
                    //SERIAL_PROTOCOLLN("PAGE(1) = 101 ");
                }
            }
            */
            //
            
            if (MENU_PAGE == 300)
            {
                lcd_cls_flg = false;
                encoderPosition = 5;
                //lcdDrawUpdate = 2;
                lcd_quick_feedback();
                currentMenu = lcd_move_y_menu;
            }
            if (MENU_PAGE == 301)
            {
                lcd_cls_flg = false;
                encoderPosition = 15;
                //lcdDrawUpdate = 2;
                lcd_quick_feedback();
                currentMenu = lcd_main_menu;
            }
            if (MENU_PAGE == 302)
            {
                lcd_cls_flg = false;
                encoderPosition = 15;
                //lcdDrawUpdate = 2;
                lcd_quick_feedback();
                currentMenu = lcd_move_x_menu;
            }
            
#if SW_MON == 1
            SERIAL_ECHOPGM("page = ");
            SERIAL_PROTOCOLLN((int)MENU_PAGE);
#endif
        }
    }
    
    //home switch
    if (READ(BTN_HOME) == 0)
    {
        delay(2);
        if (READ(BTN_HOME) != 0)
            return;

        if (PUSH_SW != 6) //down
        {
            SW_RELEASE = false; 
            PUSH_SW = 6;
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_HOME(0)");
#endif
            
            MOVE_HOME_OK = false;
            enquecommand_P(PSTR("G28"));
            return;
        }
        else if (SW_RELEASE == true)
        {
            SW_RELEASE = false;
            
#if SW_MON == 1
            SERIAL_PROTOCOLLN("SW_HOME(1)");
#endif
        }
    }
    
    if ((blocking_enc<millis()) && (READ(BTN_EN1) == 1) && (READ(BTN_EN2) == 1)
       && (READ(BTN_LEFT) == 1) && (READ(BTN_RIGHT) == 1) && (READ(BTN_HOME) == 1))
    {
        SW_RELEASE = true;
        //PUSH_SW = 0;
    }

#endif //SW_METHOD
    
///

/* origin
      uint8_t newbutton=0;
      if(READ(BTN_EN1)==0)  newbutton|=EN_A;
      if(READ(BTN_EN2)==0)  newbutton|=EN_B;
      if((blocking_enc<millis()) && (READ(BTN_ENC)==0))
          newbutton |= EN_C;
      buttons = newbutton;
*/
#else   //read it from the shift register
/*
    uint8_t newbutton=0;
    WRITE(SHIFT_LD,LOW);
    WRITE(SHIFT_LD,HIGH);
    unsigned char tmp_buttons=0;
    for(int8_t i=0;i<8;i++)
    { 
        newbutton = newbutton>>1;
        if(READ(SHIFT_OUT))
            newbutton|=(1<<7);
        WRITE(SHIFT_CLK,HIGH);
        WRITE(SHIFT_CLK,LOW);
    }
    buttons=~newbutton; //invert it, because a pressed switch produces a logical 0
*/

#endif//!NEWPANEL

/* original
    //manage encoder rotation
    uint8_t enc=0;
    
    if(buttons&EN_A)
        enc|=(1<<0);
    if(buttons&EN_B)
        enc|=(1<<1);
    
    if(enc != lastEncoderBits)
    {
        switch(enc)
        {
        case encrot0:
            if(lastEncoderBits==encrot3) {
                encoderDiff++;
                SERIAL_PROTOCOLLN("encrot0==encrot3");
            }
            else if(lastEncoderBits==encrot1) {
                encoderDiff--;
                SERIAL_PROTOCOLLN("encrot0==encrot1");
            }
            break;
        case encrot1:
            if(lastEncoderBits==encrot0) {
                encoderDiff++;
                SERIAL_PROTOCOLLN("encrot1==encrot0");
            }
            else if(lastEncoderBits==encrot2) {
                encoderDiff--;
                SERIAL_PROTOCOLLN("encrot1==encrot2");
            }
            break;
        case encrot2:
            if(lastEncoderBits==encrot1) {
                encoderDiff++;
                SERIAL_PROTOCOLLN("encrot2==encrot1");
            }
            else if(lastEncoderBits==encrot3) {
                encoderDiff--;
                SERIAL_PROTOCOLLN("encrot2==encrot3");
            }
            break;
        case encrot3:
            if(lastEncoderBits==encrot2) {
                encoderDiff++;
                SERIAL_PROTOCOLLN("encrot3==encrot2");
            }
            else if(lastEncoderBits==encrot0) {
                encoderDiff--;
                //encoderDiff = 1;
                SERIAL_PROTOCOLLN("encrot3==encrot0");
                SERIAL_PROTOCOLLN("encoderDiff= ");
                SERIAL_PROTOCOLLN((int)encoderDiff);
                
            }
            break;
        }
    }
*/    
    lastEncoderBits = enc;
    
    
    
}
#endif//ULTIPANEL

/********************************/
/** Float conversion utilities **/
/********************************/
//  convert float to string with +123.4 format
char conv[8];
char *ftostr3(const float &x)
{
  return itostr3((int)x);
}

char *itostr2(const uint8_t &x)
{
  //sprintf(conv,"%5.1f",x);
  int xx=x;
  conv[0]=(xx/10)%10+'0';
  conv[1]=(xx)%10+'0';
  conv[2]=0;
  return conv;
}

//  convert float to string with +123.4 format
char *ftostr31(const float &x)
{
  int xx=x*10;
  conv[0]=(xx>=0)?'+':'-';
  xx=abs(xx);
  conv[1]=(xx/1000)%10+'0';
  conv[2]=(xx/100)%10+'0';
  conv[3]=(xx/10)%10+'0';
  conv[4]='.';
  conv[5]=(xx)%10+'0';
  conv[6]=0;
  return conv;
}

//  convert float to string with 123.4 format
char *ftostr31ns(const float &x)
{
  int xx=x*10;
  //conv[0]=(xx>=0)?'+':'-';
  xx=abs(xx);
  conv[0]=(xx/1000)%10+'0';
  conv[1]=(xx/100)%10+'0';
  conv[2]=(xx/10)%10+'0';
  conv[3]='.';
  conv[4]=(xx)%10+'0';
  conv[5]=0;
  return conv;
}

char *ftostr32(const float &x)
{
  long xx=x*100;
  if (xx >= 0)
    conv[0]=(xx/10000)%10+'0';
  else
    conv[0]='-';
  xx=abs(xx);
  conv[1]=(xx/1000)%10+'0';
  conv[2]=(xx/100)%10+'0';
  conv[3]='.';
  conv[4]=(xx/10)%10+'0';
  conv[5]=(xx)%10+'0';
  conv[6]=0;
  return conv;
}

char *itostr31(const int &xx)
{
  conv[0]=(xx>=0)?'+':'-';
  conv[1]=(xx/1000)%10+'0';
  conv[2]=(xx/100)%10+'0';
  conv[3]=(xx/10)%10+'0';
  conv[4]='.';
  conv[5]=(xx)%10+'0';
  conv[6]=0;
  return conv;
}


char *itostr3(const int &xx)
{
  if (xx >= 100)
    conv[0]=(xx/100)%10+'0';
  else
    conv[0]=' ';
  if (xx >= 10)
    conv[1]=(xx/10)%10+'0';
  else
    conv[1]=' ';
  conv[2]=(xx)%10+'0';
  conv[3]=0;
  return conv;
}

char *itostr3left(const int &xx)
{
  if (xx >= 100)
  {
    conv[0]=(xx/100)%10+'0';
    conv[1]=(xx/10)%10+'0';
    conv[2]=(xx)%10+'0';
    conv[3]=0;
  }
  else if (xx >= 10)
  {
    conv[0]=(xx/10)%10+'0';
    conv[1]=(xx)%10+'0';
    conv[2]=0;
    //conv[3]=' ';
  }
  else
  {
    conv[0]=(xx)%10+'0';
    conv[1]=0;
    //conv[2]=' ';
    //conv[3]=' ';
  }
  return conv;
}

char *itostr4(const int &xx)
{
  if (xx >= 1000)
    conv[0]=(xx/1000)%10+'0';
  else
    conv[0]=' ';
  if (xx >= 100)
    conv[1]=(xx/100)%10+'0';
  else
    conv[1]=' ';
  if (xx >= 10)
    conv[2]=(xx/10)%10+'0';
  else
    conv[2]=' ';
  conv[3]=(xx)%10+'0';
  conv[4]=0;
  return conv;
}

//  convert float to string with 12345 format
char *ftostr5(const float &x)
{
  long xx=abs(x);
  if (xx >= 10000)
    conv[0]=(xx/10000)%10+'0';
  else
    conv[0]=' ';
  if (xx >= 1000)
    conv[1]=(xx/1000)%10+'0';
  else
    conv[1]=' ';
  if (xx >= 100)
    conv[2]=(xx/100)%10+'0';
  else
    conv[2]=' ';
  if (xx >= 10)
    conv[3]=(xx/10)%10+'0';
  else
    conv[3]=' ';
  conv[4]=(xx)%10+'0';
  conv[5]=0;
  return conv;
}

//  convert float to string with +1234.5 format
char *ftostr51(const float &x)
{
  long xx=x*10;
  conv[0]=(xx>=0)?'+':'-';
  xx=abs(xx);
  conv[1]=(xx/10000)%10+'0';
  conv[2]=(xx/1000)%10+'0';
  conv[3]=(xx/100)%10+'0';
  conv[4]=(xx/10)%10+'0';
  conv[5]='.';
  conv[6]=(xx)%10+'0';
  conv[7]=0;
  return conv;
}

//  convert float to string with +123.45 format
char *ftostr52(const float &x)
{
  long xx=x*100;
  conv[0]=(xx>=0)?'+':'-';
  xx=abs(xx);
  conv[1]=(xx/10000)%10+'0';
  conv[2]=(xx/1000)%10+'0';
  conv[3]=(xx/100)%10+'0';
  conv[4]='.';
  conv[5]=(xx/10)%10+'0';
  conv[6]=(xx)%10+'0';
  conv[7]=0;
  return conv;
}

//2014/07/08
char *ftostr31u(const float &x)
{
  long xx=x*100;
  if (xx >= 0)
    conv[0]=(xx/10000)%10+'0';
  else
    conv[0]='-';
  xx=abs(xx);
  conv[1]=(xx/1000)%10+'0';
  conv[2]=(xx/100)%10+'0';
  conv[3]='.';
  conv[4]=(xx/10)%10+'0';
  //conv[5]=(xx)%10+'0';
  conv[5]=0;
  return conv;
}

char *ftostr33(const float &x)
{
  int xx=x;	
  if (xx >= 100)
    conv[0]=(xx/100)%10+'0';
  else
    conv[0]='0';
  if (xx >= 10)
    conv[1]=(xx/10)%10+'0';
  else
    conv[1]='0';
  conv[2]=(xx)%10+'0';
  conv[3]=0;
  return conv;	
}
///



/*
// Callback for after editing PID i value
// grab the pid i value out of the temp variable; scale it; then update the PID driver
void copy_and_scalePID_i()
{
  Ki = scalePID_i(raw_Ki);
  updatePID();
}	

// Callback for after editing PID d value
// grab the pid d value out of the temp variable; scale it; then update the PID driver
void copy_and_scalePID_d()
{
  Kd = scalePID_d(raw_Kd);
  updatePID();
}	
*/
	
#endif //ULTRA_LCD
